@@ -5,10 +5,11 @@ Created on 29 Apr 2021
 '''
 from trexlib.utils.string_util import is_empty, is_not_empty
 from datetime import datetime
+from dateutil.relativedelta import relativedelta
 import logging
 
-#logger = logging.getLogger('helper')
-logger = logging.getLogger('debug')
+logger = logging.getLogger('helper')
+#logger = logging.getLogger('debug')
 
 
 '''
@@ -129,7 +130,7 @@ def update_reward_summary_with_reverted_reward(existing_reward_summary, revertin
             reward_summary_by_reward_format['amount']   = final_reward_amount
             reward_summary[reward_format]               = reward_summary_by_reward_format
             
-            if final_reward_amount==0:
+            if final_reward_amount<=0:
                 del reward_summary[reward_format]
         
     return reward_summary
@@ -140,6 +141,7 @@ def update_reward_summary_with_reverted_reward(existing_reward_summary, revertin
 customer entitled_voucher_summary format is
 {
     voucher_key : 
+                key   : xxxx,
                 label : xxxxxx,
                 image_url : xxxxxxxx,
                 redeem_info_list :    [
@@ -160,36 +162,75 @@ def update_customer_entiteld_voucher_summary_with_customer_new_voucher(customer_
     voucher_key             = merchant_voucher.key_in_str
     voucher_label           = merchant_voucher.label
     voucher_image_url       = merchant_voucher.image_public_url
+    configuration           = merchant_voucher.rebuild_configuration
     redeem_info_list        = [customer_voucher.to_redeem_info()]
     
+    logger.debug('update_customer_entiteld_voucher_summary_with_customer_new_voucher debug: voucher_key=%s', voucher_key)
+    logger.debug('update_customer_entiteld_voucher_summary_with_customer_new_voucher debug: voucher_label=%s', voucher_label)
+    
+    
     return update_customer_entiteld_voucher_summary_with_new_voucher_info(customer_entitled_voucher_summary, 
                                                                           voucher_key, 
                                                                           voucher_label, 
                                                                           voucher_image_url,
-                                                                          redeem_info_list)
+                                                                          redeem_info_list,
+                                                                          configuration,
+                                                                          )
 
 def update_customer_entiteld_voucher_summary_with_new_voucher_info(customer_entitled_voucher_summary, merchant_voucher_key, voucher_label, 
-                                                              voucher_image_url, redeem_info_list):
+                                                              voucher_image_url, redeem_info_list, configuration):
     
     if customer_entitled_voucher_summary is None:
         customer_entitled_voucher_summary = {}
         
-    voucher_summary     = customer_entitled_voucher_summary.get(merchant_voucher_key)
+    voucher_summary         = customer_entitled_voucher_summary.get(merchant_voucher_key)
+    latest_expiry_date_str  = None
+    if redeem_info_list and len(redeem_info_list)>0:
+        
+        latest_expiry_date      = get_latest_expiry_date(redeem_info_list)
+        latest_expiry_date_str  = datetime.strftime(latest_expiry_date, '%d-%m-%Y')
+    
+    logger.debug('New entitled voucher latest_expiry_date_str=%s', latest_expiry_date_str)
     
     if voucher_summary:
-        customer_entitled_voucher_summary[merchant_voucher_key]['redeem_info_list'].extend(redeem_info_list)  
+        logger.debug('Found voucher summary, thus going to add new redeem info')
+        customer_entitled_voucher_summary[merchant_voucher_key]['redeem_info_list'].extend(redeem_info_list)
+        customer_entitled_voucher_summary[merchant_voucher_key]['amount'] = len(customer_entitled_voucher_summary[merchant_voucher_key]['redeem_info_list'])
+          
     else:
+        logger.debug('Not found voucher summary, thus going to create it')
         voucher_summary = {
+                            'key'               : merchant_voucher_key,
                             'label'             : voucher_label,
+                            'configuration'     : configuration,
                             'image_url'         : voucher_image_url,
-                            'redeem_info_list'  : redeem_info_list 
+                            'redeem_info_list'  : redeem_info_list,
+                            'count'             : len(redeem_info_list), 
                             
                             }
         customer_entitled_voucher_summary[merchant_voucher_key] = voucher_summary
-
+    
+    if latest_expiry_date_str:
+        customer_entitled_voucher_summary[merchant_voucher_key]['latest_expiry_date'] = latest_expiry_date_str
+    
     return customer_entitled_voucher_summary
 
-def update_customer_entiteld_voucher_summary_after_reverted_voucher(customer_entitled_voucher_summary, reverted_customer_voucher):
+def get_latest_expiry_date(redeem_info_list):
+    latest_expiry_date = None
+    for info in redeem_info_list:
+        expiry_date_str = info.get('expiry_date')
+        expiry_date     = datetime.strptime(expiry_date_str, '%d-%m-%Y')
+        
+        if latest_expiry_date is None:
+            latest_expiry_date = datetime.strptime(expiry_date_str, '%d-%m-%Y')
+        else:
+            if expiry_date>latest_expiry_date:
+                latest_expiry_date = expiry_date
+    
+    return latest_expiry_date
+        
+
+def update_customer_entiteld_voucher_summary_after_removed_voucher(customer_entitled_voucher_summary, removed_customer_voucher):
     '''
     removed entitled voucher from customer entitled voucher summary 
     '''
@@ -197,8 +238,8 @@ def update_customer_entiteld_voucher_summary_after_reverted_voucher(customer_ent
     
     
     if customer_entitled_voucher_summary:
-        merchant_voucher_key                    = reverted_customer_voucher.entitled_voucher_key
-        redeem_code_of_reverting_voucher        = reverted_customer_voucher.redeem_code
+        merchant_voucher_key                    = removed_customer_voucher.entitled_voucher_key
+        redeem_code_of_reverting_voucher        = removed_customer_voucher.redeem_code
         
         logger.debug('merchant_voucher_key=%s', merchant_voucher_key)
         logger.debug('redeem_code_of_reverting_voucher=%s', redeem_code_of_reverting_voucher)
@@ -219,11 +260,17 @@ def update_customer_entiteld_voucher_summary_after_reverted_voucher(customer_ent
                 del customer_entitled_voucher_summary[merchant_voucher_key]
             else:
                 customer_entitled_voucher_summary[merchant_voucher_key]['redeem_info_list'] = new_redeem_info_list
+                customer_entitled_voucher_summary[merchant_voucher_key]['count']            = len(new_redeem_info_list)
+                
     return customer_entitled_voucher_summary
     
-    
+
+def update_customer_entiteld_voucher_summary_after_reverted_voucher(entitled_voucher_summary, reverted_customer_voucher):
+    return update_customer_entiteld_voucher_summary_after_removed_voucher(entitled_voucher_summary, reverted_customer_voucher)
+
 def update_customer_entiteld_voucher_summary_after_redeemed_voucher(entitled_voucher_summary, redeemed_customer_voucher):
-    return update_customer_entiteld_voucher_summary_after_reverted_voucher(entitled_voucher_summary, redeemed_customer_voucher)
+    return update_customer_entiteld_voucher_summary_after_removed_voucher(entitled_voucher_summary, redeemed_customer_voucher)
+
 
 ''' --------------- End: Update reward summary for voucher--------------'''
 
@@ -232,39 +279,44 @@ def update_customer_entiteld_voucher_summary_after_redeemed_voucher(entitled_vou
 def update_prepaid_summary_with_new_prepaid(existing_prepaid_summary, new_prepaid_summary):
     prepaid_amount              = new_prepaid_summary.get('amount')
     used_prepaid_amount         = new_prepaid_summary.get('used_amount')
+    prepaid_balance             = prepaid_amount - used_prepaid_amount
     
-    if is_empty(existing_prepaid_summary):
-        prepaid_balance         = prepaid_amount - used_prepaid_amount
-        existing_prepaid_summary = {
-                                    'amount'  : prepaid_balance,
-                                    }
-        
-    else:
-        prepaid_balance = existing_prepaid_summary.get('amount') + (prepaid_amount-used_prepaid_amount)
-        existing_prepaid_summary = {
-                                    'amount'  : prepaid_balance,
-                                    }        
+    if is_not_empty(existing_prepaid_summary):
+        prepaid_balance = existing_prepaid_summary.get('amount') + prepaid_balance
                 
+    existing_prepaid_summary = {
+                                    'amount'  : prepaid_balance,
+                                    }            
     return existing_prepaid_summary
 
 def update_prepaid_summary_with_reverted_prepaid(existing_prepaid_summary, reverted_prepaid_summary):
     prepaid_amount              = reverted_prepaid_summary.get('amount')
-    used_prepaid_amount         = reverted_prepaid_summary.get('used_amount')
+    
     
     if is_empty(existing_prepaid_summary):
         existing_prepaid_summary = {
-                                    'amount'  : .0,
+                                    'amount'  : prepaid_amount,
                                     }
         
     else:
-        prepaid_balance = existing_prepaid_summary.get('amount') - (prepaid_amount-used_prepaid_amount)
+        prepaid_balance = existing_prepaid_summary.get('amount') - prepaid_amount
         if prepaid_balance<0:
             prepaid_balance = .0
-        
+            
         existing_prepaid_summary = {
                                     'amount'  : prepaid_balance,
                                     }        
                 
     return existing_prepaid_summary
 
-''' --------------- End: Update reward summary for prepaid--------------'''
\ No newline at end of file
+''' --------------- End: Update reward summary for prepaid--------------'''
+
+
+
+
+
+
+
+
+
+    
\ No newline at end of file
