@@ -18,6 +18,7 @@ from trexmodel import conf, program_conf
 from trexmodel.models.datastore.system_models import Tagging
 from trexmodel.models.datastore.model_decorators import model_transactional
 from trexlib.utils.common.common_util import sort_dict_list
+from trexmodel.program_conf import LOYALTY_PACKAGE_LITE, LOYALTY_PRODUCT
 
 logger = logging.getLogger('model')
 #logger = logging.getLogger('debug')
@@ -59,27 +60,42 @@ class MerchantAcct(MerchantMin):
     dashboard_stat_figure                   = ndb.JsonProperty()
     currency_code                           = ndb.StringProperty(required=False, default='MYR')
     locale                                  = ndb.StringProperty(required=False, default='en_MY')
+    timezone                                = ndb.StringProperty(required=False, default='Asia/Kuala_Lumpur')
     api_key                                 = ndb.StringProperty(required=False)
     
+    account_plan                            = ndb.JsonProperty()
+    outlet_count                            = ndb.IntegerProperty(default=0)
     
-    published_program_configuration         = ndb.JsonProperty()
-    published_voucher_configuration         = ndb.JsonProperty()
-    membership_configuration                = ndb.JsonProperty()
-    tier_membership_configuration           = ndb.JsonProperty()
+    published_program_configuration                 = ndb.JsonProperty()
+    published_voucher_configuration                 = ndb.JsonProperty()
+    published_redemption_catalogue_configuration    = ndb.JsonProperty()
+    
+    membership_configuration                        = ndb.JsonProperty()
+    tier_membership_configuration                   = ndb.JsonProperty()
     
     reward_naming_configuration             = ndb.JsonProperty()
     
     prepaid_configuration                   = ndb.JsonProperty()
     
+    lucky_draw_configuration                = ndb.JsonProperty(required=False)
+    
     product_modifier_configuration          = ndb.JsonProperty()
     
+    program_settings                        = ndb.JsonProperty()
+    
     stat_figure_update_interval_in_minutes  = conf.MERCHANT_STAT_FIGURE_UPDATE_INTERVAL_IN_MINUTES
     stat_figure_update_datetime_format      = '%d-%m-%Y %H:%M:%S'
     
-    dict_properties = ['company_name', 'brand_name', 'contact_name', 'business_reg_no', 'mobile_phone', 'office_phone', 'fax_phone', 'email', 'account_code', 'country',
+    dict_properties = ['company_name', 'brand_name', 'contact_name', 'business_reg_no', 'mobile_phone', 
+                       'office_phone', 'fax_phone', 'email', 'account_code', 'country',
                        'registered_datetime', 'modified_datetime', 'plan_start_date', 'plan_end_date', 'currency_code', 
+                       'timezone',
                        'published_program_configuration', 'published_voucher_configuration', 'membership_configuration', 
-                       'tier_membership_configuration', 'prepaid_configuration', 'product_modifier_configuration']
+                       'tier_membership_configuration', 'prepaid_configuration', 'lucky_draw_configuration', 'product_modifier_configuration',
+                       'dashboard_stat_figure', 'program_settings', 'is_tier_membership_configured', 'website', 
+                       'product_package', 'loyalty_package','account_plan','outlet_count', 'outlet_limit',
+                       'published_redemption_catalogue_configuration',
+                       ]
     
     
     def to_login_dict(self):
@@ -88,13 +104,133 @@ class MerchantAcct(MerchantMin):
                 'plan_end_date'     : self.plan_end_date,
                 'currency_code'     : self.currency_code,
                 'country'           : self.country,
+                'account_plan'      : self.account_plan,
                 }
     
-       
+    @property
+    def product_package(self):
+        if self.account_plan:
+            if self.account_plan.get('product_package'):
+                return self.account_plan.get('product_package')
+        
+        return [LOYALTY_PRODUCT]
     
+    @property
+    def loyalty_package(self):
+        if self.account_plan:
+            if self.account_plan.get('loyalty_package'):
+                return self.account_plan.get('loyalty_package')
+        
+        return LOYALTY_PACKAGE_LITE
+    
+    @property
+    def outlet_limit(self):
+        if self.account_plan:
+            if self.account_plan.get('outlet_limit'):
+                return self.account_plan.get('outlet_limit')
+        
+        return 1
+    
+    @property
+    def program_configuration_list(self):
+        if is_not_empty(self.published_program_configuration):
+            return  self.published_program_configuration.get('programs')
+        else:
+            return []
+    
+    
+    @property
+    def is_tier_membership_configured(self):
+        return is_not_empty(self.tier_membership_configuration)
+    
+    @property
+    def image_default_base_url(self):
+        return program_conf.IMAGE_BASE_URL
+    
+    @property
+    def days_of_return_policy(self):
+        return self.program_settings.get('days_of_return_policy') or MerchantAcct.default_program_settings().get('days_of_return_policy')
+    
+    @property
+    def days_of_repeat_purchase_measurement(self):
+        return self.program_settings.get('days_of_repeat_purchase_measurement') or MerchantAcct.default_program_settings().get('days_of_repeat_purchase_measurement')
+    
+    @property
+    def membership_renew_advance_day(self):
+        return self.program_settings.get('membership_renew_advance_day') or MerchantAcct.default_program_settings().get('membership_renew_advance_day')
+    
+    @property
+    def membership_renew_late_day(self):
+        return self.program_settings.get('membership_renew_late_day') or MerchantAcct.default_program_settings().get('membership_renew_late_day')
+    
+    
+    
+    @property
+    def lucky_draw_program_count(self):
+        if self.lucky_draw_configuration:
+            if self.lucky_draw_configuration.get('count'):
+                return self.lucky_draw_configuration.get('count')
+        return 0
+    
+    @property
+    def lucky_draw_ticket_is_recurring_scheme(self):
+        if self.lucky_draw_configuration.get('settings')  and self.lucky_draw_configuration.get('settings').get('is_recurring_scheme'):
+            return self.lucky_draw_configuration.get('settings').get('is_recurring_scheme')
+        else:
+            return False
+            
+    @property
+    def lucky_draw_ticket_spending_currency(self):
+        if self.lucky_draw_configuration.get('settings') and self.lucky_draw_configuration.get('settings').get('spending_currency'):
+            return self.lucky_draw_configuration.get('settings').get('spending_currency')
+        else:
+            return 0
+    
+    @property
+    def lucky_draw_ticket_limit_type(self):
+        if self.lucky_draw_configuration.get('settings'):
+            return self.lucky_draw_configuration.get('settings').get('ticket_limit_type') or program_conf.REWARD_LIMIT_TYPE_NO_LIMIT
+        else:
+            return program_conf.REWARD_LIMIT_TYPE_NO_LIMIT
+            
+    
+    @property
+    def lucky_draw_ticket_limit_amount(self):
+        if self.lucky_draw_configuration.get('settings'):
+            return self.lucky_draw_configuration.get('settings').get('ticket_limit_amount')
+        
+        return 0
+    
+    @property
+    def lucky_draw_ticket_expiry_date_length_in_day(self):
+        if self.lucky_draw_configuration.get('settings'):
+            return self.lucky_draw_configuration.get('settings').get('ticket_expiry_date_length_in_day')
+        
+        return 0
+    
+    @property
+    def lucky_draw_ticket_image_url(self):
+        if self.lucky_draw_configuration.get('settings'):
+            return self.lucky_draw_configuration['settings']['ticket_image_url']
     
     @staticmethod
-    def update_details(merchant_acct, company_name=None, brand_name=None, business_reg_no=None, contact_name=None, email=None, mobile_phone=None, office_phone=None, currency_code=None, country=None):
+    def default_program_settings():
+        return {
+                'days_of_return_policy'                 : 3,
+                'days_of_repeat_purchase_measurement'   : 7,
+                'membership_renew_advance_day'          : 7,
+                'membership_renew_late_day'             : 30,
+                
+                }
+        
+    
+    @staticmethod
+    def update_details(merchant_acct, company_name=None, brand_name=None, business_reg_no=None, contact_name=None, 
+                       email=None, mobile_phone=None, office_phone=None, currency_code=None,
+                       country=None, timezone=None, website=None):
+        
+        
+        
         merchant_acct.company_name      = company_name
         merchant_acct.brand_name        = brand_name
         merchant_acct.business_reg_no   = business_reg_no
@@ -104,6 +240,9 @@ class MerchantAcct(MerchantMin):
         merchant_acct.office_phone      = office_phone
         merchant_acct.currency_code     = currency_code
         merchant_acct.country           = country
+        merchant_acct.timezone          = timezone
+        merchant_acct.website           = website
+        
         merchant_acct.put()
     
     @staticmethod
@@ -269,7 +408,7 @@ class MerchantAcct(MerchantMin):
             
         self.put()
         
-    def update_published_voucher(self, voucher_configuration):
+    def add_voucher(self, voucher_configuration):
         if is_empty(self.published_voucher_configuration):
             self.published_voucher_configuration = {
                                                 'vouchers'  :[voucher_configuration],
@@ -295,6 +434,20 @@ class MerchantAcct(MerchantMin):
             
         self.put() 
         
+    def remove_voucher(self, voucher_key):
+        existing_vouchers_list = self.published_voucher_configuration['vouchers']
+        
+        new_vouchers_list = []
+        for voucher in existing_vouchers_list:
+            if voucher.get('voucher_key') != voucher_key:
+                new_vouchers_list.append(voucher)
+            
+        
+        self.published_voucher_configuration['vouchers']    = new_vouchers_list
+        self.published_voucher_configuration['count']       = len(new_vouchers_list)
+            
+        self.put()    
+        
     def update_prepaid_program(self, prepaid_configuration):
         if self.prepaid_configuration is None or len(self.prepaid_configuration)==0:
             self.prepaid_configuration = {
@@ -321,6 +474,74 @@ class MerchantAcct(MerchantMin):
             
         self.put()
         
+    def update_lucky_draw_program(self, lucky_draw_configuration):
+        logger.debug('update_lucky_draw_program debug: lucky_draw_configuration = %s', lucky_draw_configuration)
+        if self.lucky_draw_configuration is None or len(self.lucky_draw_configuration)==0:
+            self.lucky_draw_configuration = {
+                                                'programs'  :[lucky_draw_configuration],
+                                                'count'     : 1,
+                                                } 
+                                                
+        existing_lucky_draw_list  = self.lucky_draw_configuration.get('programs')
+        
+        program_key = lucky_draw_configuration.get('program_key')
+        
+        index = 0
+        for v in existing_lucky_draw_list:
+            if v.get('program_key') == program_key:
+                existing_lucky_draw_list.pop(index)
+            
+            index = index+1
+        
+        existing_lucky_draw_list.append(lucky_draw_configuration)
+        
+        self.lucky_draw_configuration['programs']        = existing_lucky_draw_list
+        self.lucky_draw_configuration['count']           = len(existing_lucky_draw_list) 
+            
+        self.put()  
+    
+    def update_published_redemption_catalogue(self, catalogue_configuration):
+        if is_empty(self.published_redemption_catalogue_configuration):
+            self.published_redemption_catalogue_configuration = {
+                                                'catalogues'    :[catalogue_configuration],
+                                                'count'         : 1,
+                                                } 
+                                            
+        else:
+            existing_catalogue_list  = self.published_redemption_catalogue_configuration.get('catalogues')
+            
+            catalogue_key = catalogue_configuration.get('catalogue_key')
+            
+            index = 0
+            for v in existing_catalogue_list:
+                if v.get('catalogue_key') == catalogue_key:
+                    existing_catalogue_list.pop(index)
+                
+                index = index+1
+            
+            existing_catalogue_list.append(catalogue_configuration)
+            
+            self.published_redemption_catalogue_configuration['catalogues']  = existing_catalogue_list
+            self.published_redemption_catalogue_configuration['count']       = len(existing_catalogue_list) 
+            
+        self.put()   
+        
+    def update_lucky_draw_program_settings(self, lucky_draw_program_settings):
+        logger.debug('update_lucky_draw_program_settings debug: lucky_draw_program_settings = %s', lucky_draw_program_settings)
+        if self.lucky_draw_configuration is None or len(self.lucky_draw_configuration)==0:
+            self.lucky_draw_configuration = {'settings':{}}
+            
+        self.lucky_draw_configuration['settings'] = lucky_draw_program_settings
+        self.put()
+        
+    def update_lucky_draw_ticket_image(self, ticket_image_url):
+        if self.lucky_draw_configuration is None or len(self.lucky_draw_configuration)==0:
+            self.lucky_draw_configuration = {'settings':{}}
+            
+        self.lucky_draw_configuration['settings']['ticket_image_url'] = ticket_image_url
+        self.put()    
+        
+        
     def update_product_modifier(self, product_modifier_configuration):
         if self.product_modifier_configuration is None or len(self.product_modifier_configuration)==0:
             self.product_modifier_configuration = {
@@ -457,19 +678,35 @@ class MerchantAcct(MerchantMin):
             
         self.put() 
         
-    def remove_archieve_voucher(self, archieve_voucher_key):
-        existing_vouchers_list = self.published_voucher_configuration['vouchers']
+    
         
-        new_vouchers_list = []
-        for voucher in existing_vouchers_list:
-            if voucher.get('code') != archieve_voucher_key:
-                new_vouchers_list.append(voucher)
+    def remove_redeption_catalogue(self, catalogue_key):
+        existing_catalogues_list = self.published_redemption_catalogue_configuration['catalogues']
+        
+        new_catalogues_list = []
+        for catalogue in existing_catalogues_list:
+            if catalogue.get('catalogue_key') != catalogue_key:
+                new_catalogues_list.append(catalogue)
             
         
-        self.published_voucher_configuration['vouchers']    = new_vouchers_list
-        self.published_voucher_configuration['count']       = len(new_vouchers_list)
+        self.published_redemption_catalogue_configuration['catalogues']      = new_catalogues_list
+        self.published_redemption_catalogue_configuration['count']           = len(new_catalogues_list)
             
-        self.put()
+        self.put()    
+        
+    def remove_archieve_redemption_catalogue(self, archieve_redemption_catalogue_key):
+        existing_catalogues_list = self.published_redemption_catalogue_configuration['catalogues']
+        
+        new_catalogues_list = []
+        for catalogue in existing_catalogues_list:
+            if catalogue.get('catalogue_key') != archieve_redemption_catalogue_key:
+                new_catalogues_list.append(catalogue)
+            
+        
+        self.published_redemption_catalogue_configuration['catalogues']  = new_catalogues_list
+        self.published_redemption_catalogue_configuration['count']       = len(new_catalogues_list)
+            
+        self.put()    
         
     def remove_archieve_basic_membership(self, archieve_membership_key):
         self.flush_dirty_membership_configuration()
@@ -538,6 +775,43 @@ class MerchantAcct(MerchantMin):
                 
             self.put() 
             
+    def remove_lucky_draw_program_configuration(self, program_key_to_remove):
+        
+        logger.debug('remove_prepaid_program_configuration: program_key_to_remove=%s', program_key_to_remove)
+        if self.lucky_draw_configuration and self.lucky_draw_configuration.get('programs'):
+            existing_programs_list  = self.lucky_draw_configuration['programs']
+            program_count           = len(existing_programs_list)
+            
+            logger.debug('program_count before remove=%s', program_count)
+            
+            index = 0
+            found = False
+            
+            for program in existing_programs_list:
+                
+                logger.debug('program_key=%s', program.get('program_key'))
+                
+                is_same_program_key = program.get('program_key') == program_key_to_remove
+                
+                logger.debug('is_same_program_key=%s', is_same_program_key)
+                
+                if is_same_program_key:
+                    existing_programs_list.pop(index)
+                    found = True
+                    logger.debug('Found program to be remove')
+                    
+                index = index+1
+            
+            if found:
+                program_count = len(existing_programs_list)
+                
+                logger.debug('program_count after remove=%s', program_count)
+                
+                self.lucky_draw_configuration['programs']    = existing_programs_list
+                self.lucky_draw_configuration['count']       = program_count
+                    
+                self.put()         
+            
     def remove_product_modifier_configuration(self, modifier_key_to_remove):
         
         logger.debug('remove_product_modifier_configuration: modifier_key_to_remove=%s', modifier_key_to_remove)
@@ -610,11 +884,20 @@ class MerchantAcct(MerchantMin):
         return None
     
     @staticmethod
-    def create(company_name=None, brand_name=None, contact_name=None, email=None, mobile_phone=None, office_phone=None, plan_start_date=None, plan_end_date=None, 
-               account_code=None, currency_code=None, country=None):
+    def create(company_name=None, brand_name=None, contact_name=None, email=None, mobile_phone=None, office_phone=None, 
+               plan_start_date=None, plan_end_date=None,  
+               account_code=None, currency_code=None, country=None, timezone=None, website=None,
+               product_package=None,loyalty_package=None, outlet_limit=1
+               ):
         
         if account_code is None:
             account_code    = "%s-%s-%s-%s" % (random_number(4),random_number(4),random_number(4),random_number(4))
+        
+        account_plan = {
+                        'product_package'   : product_package.split(','),
+                        'loyalty_package'   : loyalty_package,
+                        'outlet_limit'      : outlet_limit,
+                        }
             
         merchant_acct   = MerchantAcct(
                                        company_name     = company_name,
@@ -623,17 +906,55 @@ class MerchantAcct(MerchantMin):
                                        email            = email,
                                        mobile_phone     = mobile_phone,
                                        office_phone     = office_phone,
+                                       account_plan     = account_plan,
                                        plan_start_date  = plan_start_date, 
                                        plan_end_date    = plan_end_date,
                                        currency_code    = currency_code,
-                                       country          = country,    
+                                       country          = country, 
+                                       timezone         = timezone,   
+                                       website          = website,
                                        api_key          = random_string(24),
+                                       program_settings = MerchantAcct.default_program_settings(),
+                                       
+                                       
                                        )
         
         logging.debug('account_code=%s', account_code)
         
         merchant_acct.account_code = account_code
         
+        merchant_acct.put()
+        
+        return merchant_acct
+    
+    @staticmethod
+    def update(merchant_acct, company_name=None, brand_name=None, contact_name=None, email=None, mobile_phone=None, office_phone=None, 
+               plan_start_date=None, plan_end_date=None,  
+               currency_code=None, country=None, timezone=None, website=None,
+               product_package=None,loyalty_package=None, outlet_limit=1,
+               ):
+        
+        account_plan = {
+                        'product_package': product_package.split(','),
+                        'loyalty_package': loyalty_package,
+                        'outlet_limit'   : outlet_limit, 
+                        }
+        
+        merchant_acct.company_name     = company_name
+        merchant_acct.brand_name       = brand_name    
+        merchant_acct.contact_name     = contact_name
+        merchant_acct.email            = email
+        merchant_acct.mobile_phone     = mobile_phone
+        merchant_acct.office_phone     = office_phone
+        merchant_acct.plan_start_date  = plan_start_date 
+        merchant_acct.plan_end_date    = plan_end_date
+        merchant_acct.currency_code    = currency_code
+        merchant_acct.country          = country
+        merchant_acct.timezone         = timezone
+        merchant_acct.website          = website
+        merchant_acct.account_plan     = account_plan
+        
+        
         merchant_acct.put()
         
         return merchant_acct
@@ -664,6 +985,34 @@ class MerchantAcct(MerchantMin):
             
         
         start_transaction(self)
+    
+    def _list_data_import_giveaway_programs(self, target_reward_format):
+        published_program_configuration = self.published_program_configuration
+        
+        data_import_giveaway_programs = []
+        
+        for program in published_program_configuration.get('programs'):
+            if program.get('giveaway_method') == program_conf.PROGRAM_REWARD_GIVEAWAY_METHOD_SYSTEM:
+                logger.debug('giveaway system condition=%s', program.get('program_settings').get('giveaway_system_settings').get('giveaway_system_condition'))
+                if program.get('program_settings').get('giveaway_system_settings').get('giveaway_system_condition') == program_conf.GIVEAWAY_SYSTEM_CONDITION_DATA_IMPORT:
+                    logger.debug('giveaway reward_format=%s', program.get('reward_format'))
+                    if program.get('reward_format') == target_reward_format:
+                        data_import_giveaway_programs.append({
+                                                            'program_key'   : program.get('program_key'),
+                                                            'label'         : program.get('label'),
+                                                            })
+                    
+        return data_import_giveaway_programs
+    
+    def list_data_import_giveaway_prepaid_programs(self):
+        return self._list_data_import_giveaway_programs(program_conf.REWARD_FORMAT_PREPAID)
+    
+    def list_data_import_giveaway_point_programs(self):
+        return self._list_data_import_giveaway_programs(program_conf.REWARD_FORMAT_POINT)
+    
+    def list_data_import_giveaway_stamp_programs(self):
+        return self._list_data_import_giveaway_programs(program_conf.REWARD_FORMAT_STAMP)
+        
         
 class MerchantSentEmail(SentEmail):
     '''
@@ -696,11 +1045,13 @@ class Outlet(BusinessEntity, FullTextSearchable):
     assigned_catalogue_key          = ndb.StringProperty(required=False)
     assigned_dinning_table_list     = ndb.JsonProperty()
     
+    show_dinning_table_occupied     = ndb.BooleanProperty(required=False, default=False)    
+    
     fulltextsearch_field_name   = 'name'
     
     dict_properties         = ['key', 'id', 'company_name', 'business_reg_no', 'name', 'address', 'office_phone', 
                                 'fax_phone', 'email', 'business_hour', 'is_physical_store', 'assigned_catalogue_key', 'assigned_dinning_table_list',
-                                'service_charge_settings', 'assigned_tax_setup',
+                                'service_charge_settings', 'assigned_tax_setup', 
                                 'geo_location', 'created_datetime']
     
     @property
@@ -709,7 +1060,7 @@ class Outlet(BusinessEntity, FullTextSearchable):
     
     @property
     def merchant_acct_key(self):
-        return self.key.parent().urlsafe().decode('utf-8')
+        return self.key.parent().urlsafe()
     
     @property
     def outlet_key(self):
@@ -784,12 +1135,21 @@ class Outlet(BusinessEntity, FullTextSearchable):
         outlet = Outlet.fetch(outlet_key)
         if outlet:
             outlet.assigned_dinning_table_list = []
-            outlet.put()                
+            outlet.put()
+            
+    @staticmethod
+    def assign_dinning_table_control_to_outlet(outlet_key, show_dinning_table_occupied):
+        outlet = Outlet.fetch(outlet_key)
+        if outlet:
+            outlet.show_dinning_table_occupied = show_dinning_table_occupied
+            outlet.put()                        
     
     @staticmethod
     def create(merchant_acct=None,name=None, company_name=None, business_reg_no=None, address=None, email=None, fax_phone=None, 
                office_phone=None, business_hour=None, geo_location=None, is_physical_store=True, id=None):
         
+        total_count = Outlet.count_by_merchant_account(merchant_acct)
+        
         outlet   = Outlet(
                             parent              = merchant_acct.create_ndb_key(),
                             name                = name,
@@ -807,6 +1167,11 @@ class Outlet(BusinessEntity, FullTextSearchable):
         
         outlet.put()
         
+        
+        
+        merchant_acct.outlet_count= total_count + 1
+        merchant_acct.put()
+        
         return outlet
     
     @staticmethod
@@ -871,30 +1236,33 @@ class MerchantUser(UserMin, FullTextSearchable):
     '''
     parent is MerchantAcct
     '''
-    username                = ndb.StringProperty(required=True)
-    permission              = ndb.JsonProperty()
-    is_admin                = ndb.BooleanProperty(required=True, default=False)
-    basic_auth_token        = ndb.StringProperty(required=False)
+    username                        = ndb.StringProperty(required=True)
+    granted_outlets_search_list     = ndb.StringProperty(required=False, default="")
+    permission                      = ndb.JsonProperty()
+    is_admin                        = ndb.BooleanProperty(required=True, default=False)
+    basic_auth_token                = ndb.StringProperty(required=False)
     
     dict_properties         = ['user_id', 'name', 'username', 'permission', 'granted_outlet', 'granted_access',
                                 'created_datetime', 'active', 'is_admin', 'basic_auth_token', 
                                 'is_super_user', 'is_admin_user', 'is_merchant_user', 'merchant_acct_key']
     
-    fulltextsearch_field_name   = 'name'
+    fulltextsearch_field_name           = 'name'
+    fulltextsearch_field_to_lowercase   = True
     
     def to_login_dict(self):
         return {
-                'user_id'           : self.user_id,
-                'username'          : self.username,
-                'name'              : self.name,
-                'permission'        : self.permission,
-                'granted_outlet'    : self.granted_outlet,
-                'granted_access'    : self.granted_access,
-                'active'            : self.active,
-                'is_admin'          : self.is_admin,
-                'is_admin_user'     : self.is_admin_user,
-                'is_merchant_user'  : self.is_merchant_user,
-                'merchant_acct_key' : self.merchant_acct_key,
+                'key'                   : self.key_in_str,
+                'user_id'               : self.user_id,
+                'username'              : self.username,
+                'name'                  : self.name,
+                'permission'            : self.permission,
+                'granted_outlet'        : self.granted_outlet,
+                'granted_access'        : self.granted_access,
+                'active'                : self.active,
+                'is_admin'              : self.is_admin,
+                'is_admin_user'         : self.is_admin_user,
+                'is_merchant_user'      : self.is_merchant_user,
+                'merchant_acct_key'     : self.merchant_acct_key,
                 
                 }
     
@@ -983,6 +1351,8 @@ class MerchantUser(UserMin, FullTextSearchable):
             
         merchant_user.is_admin = is_admin
         merchant_user.permission = {'granted_access': access_permission, 'granted_outlet': outlet_permission}
+        merchant_user.granted_outlets_search_list =' '.join(outlet_permission)
+        
         merchant_user.put()
     
     @staticmethod
@@ -995,10 +1365,11 @@ class MerchantUser(UserMin, FullTextSearchable):
         if check_unique_merchant_user is None:
             user_id = generate_user_id()
             created_user = MerchantUser(
-                                parent = merchant_acct.create_ndb_key(),
-                                user_id=user_id, 
-                                name=name, 
-                                username=username
+                                parent      = merchant_acct.create_ndb_key(),
+                                user_id     = user_id, 
+                                name        = name, 
+                                username    = username,
+                                #country     = merchant_acct.country,
                                 )
             
             hashed_password = hash_password(user_id, password)
@@ -1041,20 +1412,65 @@ class MerchantUser(UserMin, FullTextSearchable):
     
     
     @staticmethod
-    def search_by_merchant_account(name=None, username=None,
+    def search_by_merchant_account(merchant_acct, name=None, username=None, assigned_outlet=None, 
                                  offset=0, start_cursor=None, limit=model_conf.MAX_FETCH_RECORD):
         
-        search_text_list = None
-        query = MerchantUser.query()
+        
+        query = MerchantUser.query(ancestor=merchant_acct.create_ndb_key())
         
         if is_not_empty(name):
             search_text_list = name.split(' ')
             
+            total_count                         = MerchantUser.full_text_count(search_text_list, query, conf.MAX_FETCH_RECORD_FULL_TEXT_SEARCH)
+        
+            (search_results, next_cursor)       = MerchantUser.full_text_search(search_text_list, query, offset=offset, 
+                                                                       start_cursor=start_cursor, return_with_cursor=True, 
+                                                                       limit=limit)
+            
+            return (search_results, total_count, next_cursor)
+        
+        
         elif is_not_empty(username):
             query = query.filter(MerchantUser.username==username)
             
+            
+            total_count                         = MerchantUser.count_with_condition_query(
+                                                                    query, 
+                                                                    limit=limit)
         
+            (search_results, next_cursor)       = MerchantUser.list_all_with_condition_query(
+                                                                    query, 
+                                                                    offset=offset, 
+                                                                    start_cursor=start_cursor, 
+                                                                    return_with_cursor=True, 
+                                                                    limit=limit)
+            
+            return (search_results, total_count, next_cursor)
+            
+        elif is_not_empty(assigned_outlet):
+            #query = query.filter(MerchantUser.granted_outlets_search_list==assigned_outlet)
+            query = MerchantUser.query(
+                                    
+                                        MerchantUser.granted_outlets_search_list==assigned_outlet,
+                                    )
+            
+            
+            
         
+            total_count                         = MerchantUser.count_with_condition_query(
+                                                                    query, 
+                                                                    limit=limit)
+        
+            (search_results, next_cursor)       = MerchantUser.list_all_with_condition_query(
+                                                                    query, 
+                                                                    offset=offset, 
+                                                                    start_cursor=start_cursor, 
+                                                                    return_with_cursor=True, 
+                                                                    limit=limit)
+            
+            return (search_results, total_count, next_cursor)   
+        
+        '''    
         total_count                         = MerchantUser.full_text_count(search_text_list, query, conf.MAX_FETCH_RECORD_FULL_TEXT_SEARCH)
         
         (search_results, next_cursor)       = MerchantUser.full_text_search(search_text_list, query, offset=offset, 
@@ -1062,6 +1478,9 @@ class MerchantUser(UserMin, FullTextSearchable):
                                                                    limit=limit)
         
         return (search_results, total_count, next_cursor)
+        '''
+         
+            
     
     
 class MerchantTagging(Tagging):    
@@ -1093,22 +1512,23 @@ class ReceiptSetup(BaseNModel,DictModel):
     dict_properties = ['receipt_header_settings', 'receipt_footer_settings']
     
     @staticmethod
-    def update(merchant_acct, receipt_header_settings={}, receipt_footer_settings={}):
-        receipt_setup = ReceiptSetup.get_by_merchant_acct(merchant_acct)
-        
-        if receipt_setup:
-        
-            receipt_setup.receipt_header_settings      = receipt_header_settings
-            receipt_setup.receipt_footer_settings      = receipt_footer_settings
-            
-        else:
-            receipt_setup = ReceiptSetup(
+    def create(merchant_acct, receipt_header_settings={}, receipt_footer_settings={}):
+        receipt_setup = ReceiptSetup(
                                         parent                  = merchant_acct.create_ndb_key(),
                                         receipt_header_settings = receipt_header_settings,    
                                         receipt_footer_settings = receipt_footer_settings,
                                         )
         receipt_setup.put() 
         
+        return receipt_setup
+    
+    @staticmethod
+    def update(receipt_setup, receipt_header_settings={}, receipt_footer_settings={}):
+        receipt_setup.receipt_header_settings      = receipt_header_settings
+        receipt_setup.receipt_footer_settings      = receipt_footer_settings
+            
+        receipt_setup.put() 
+        
         return receipt_setup   
     
     @staticmethod    
