@@ -22,12 +22,14 @@ logger = logging.getLogger('model')
 def convert_to_serializable_value(val, none_as_empty_string=False, gmt=0, 
                                   datetime_format=lib_conf.DEFAULT_DATETIME_FORMAT, 
                                   date_format=lib_conf.DEFAULT_DATE_FORMAT, 
-                                  time_format=lib_conf.DEFAULT_TIME_FORMAT):
+                                  time_format=lib_conf.DEFAULT_TIME_FORMAT,
+                                  excluded_dict_properties=None
+                                  ):
     
     if isinstance(val, (list, tuple)):
         _list = []
         for item in val:
-            _list.append(convert_to_serializable_value(item))
+            _list.append(convert_to_serializable_value(item, excluded_dict_properties=excluded_dict_properties))
         value = _list
     elif isinstance(val, float):
         value = format_float(val)
@@ -61,25 +63,37 @@ def convert_to_serializable_value(val, none_as_empty_string=False, gmt=0,
 def render_dict_value(val, full=False, show_key=True, gmt=0, 
                       datetime_format=lib_conf.DEFAULT_DATETIME_FORMAT, 
                       date_format=lib_conf.DEFAULT_DATE_FORMAT, 
-                      time_format=lib_conf.DEFAULT_TIME_FORMAT):
-    #logger.debug('---render_dict_value---')
+                      time_format=lib_conf.DEFAULT_TIME_FORMAT,
+                      excluded_dict_properties=None,
+                      deep_level=99
+                      ):
+    #logger.debug('---render_dict_value---, deep_level=%d', deep_level)
     value = None
-    if isinstance(val, DictModel):
-        value = val.to_dict(full=full, show_key=show_key, gmt=gmt, date_format=date_format, datetime_format=datetime_format, time_format=time_format)
-
-    elif isinstance(val, ndb.Model):
-        value = val.key.urlsafe()
-
-    elif isinstance(val, (list, tuple)):
-        _list = []
-        for item in val:
-            _list.append(render_dict_value(item, full=full, show_key=show_key, gmt=gmt, date_format=date_format, datetime_format=datetime_format, time_format=time_format))
-        value = _list
-
-    elif isinstance(val, DictObject):
-        value = val.to_dict(gmt=gmt)
-    else:
-        value = convert_to_serializable_value(val, gmt=gmt, date_format=date_format, datetime_format=datetime_format, time_format=time_format)
+    if deep_level>0:
+        if isinstance(val, DictModel):
+            
+            value = val.to_dict(full=full, show_key=show_key, gmt=gmt, 
+                                date_format=date_format, datetime_format=datetime_format, time_format=time_format, 
+                                excluded_dict_properties=excluded_dict_properties, deep_level=deep_level-1)
+    
+        elif isinstance(val, ndb.Model):
+            value = val.key.urlsafe()
+    
+        elif isinstance(val, (list, tuple)):
+            _list = []
+            for item in val:
+                _list.append(render_dict_value(
+                                item, full=full, show_key=show_key, gmt=gmt, 
+                                date_format=date_format, datetime_format=datetime_format, time_format=time_format, 
+                                excluded_dict_properties=excluded_dict_properties,
+                                deep_level=deep_level-1
+                                ))
+            value = _list
+    
+        elif isinstance(val, DictObject):
+            value = val.to_dict(gmt=gmt, excluded_dict_properties=excluded_dict_properties, deep_level=deep_level-1)
+        else:
+            value = convert_to_serializable_value(val, gmt=gmt, date_format=date_format, datetime_format=datetime_format, time_format=time_format, excluded_dict_properties=excluded_dict_properties)
     return value
 
 class DictObject(object):
@@ -98,10 +112,10 @@ class DictObject(object):
     def properties(self):
         return self.__dict__
 
-    def to_dict(self, full=True, show_key=True, dict_properties=None, gmt=lib_conf.DEFAULT_GMT, 
-                        datetime_format=lib_conf.DEFAULT_DATETIME_FORMAT, 
-                        date_format=lib_conf.DEFAULT_DATE_FORMAT, 
-                        time_format=lib_conf.DEFAULT_TIME_FORMAT):
+    def to_dict(self, full=True, show_key=True, dict_properties=None, excluded_dict_properties=None, gmt=lib_conf.DEFAULT_GMT, 
+                        datetime_format=lib_conf.DEFAULT_DATETIME_FORMAT, date_format=lib_conf.DEFAULT_DATE_FORMAT, time_format=lib_conf.DEFAULT_TIME_FORMAT, 
+                        deep_level=99,
+                        ):
         logger.info('calling DictObject.to_dict')
         result = {}
         #logger.debug('properties = %s, object = %s', self.properties(), self)
@@ -116,8 +130,11 @@ class DictObject(object):
                     #logger.debug('attribute=%s', p)
                     result[p] = render_dict_value(val, full=full, show_key=show_key, gmt=gmt, 
                                                   datetime_format=datetime_format, 
-                                                  date_format=datetime_format, 
-                                                  time_format=time_format)
+                                                  date_format=date_format, 
+                                                  time_format=time_format,
+                                                  excluded_dict_properties=excluded_dict_properties,
+                                                  deep_level=deep_level,
+                                                  )
         return result
     
 class DictModel(ndb.Model):
@@ -135,11 +152,15 @@ class DictModel(ndb.Model):
 
     def put(self, **kwargs):
         super(DictModel, self).put(**kwargs)
+        
+    def __str__(self):
+        return '%s' % self.to_dict(deep_level=1) 
 
-    def to_dict(self, full=False, show_key=True, dict_properties=None, gmt=lib_conf.DEFAULT_GMT, 
+    def to_dict(self, full=False, show_key=True, dict_properties=None, excluded_dict_properties=None, gmt=lib_conf.DEFAULT_GMT, 
                         datetime_format=lib_conf.DEFAULT_DATETIME_FORMAT, 
-                        date_format=lib_conf.DEFAULT_DATE_FORMAT, 
-                        time_format=lib_conf.DEFAULT_TIME_FORMAT):
+                        date_format=lib_conf.DEFAULT_DATE_FORMAT,time_format=lib_conf.DEFAULT_TIME_FORMAT,
+                        deep_level=99,
+                        ):
         result = {}
         logger.debug('DictModel: full=%s', full)
         logger.debug('DictModel: show_key=%s', show_key)
@@ -158,6 +179,10 @@ class DictModel(ndb.Model):
 
         if _dict_properties:
             for p in _dict_properties:
+                if excluded_dict_properties is not None:
+                    if p in excluded_dict_properties:
+                        continue
+                    
                 if p=='key':
                     if result.get('key'):
                         continue
@@ -178,7 +203,7 @@ class DictModel(ndb.Model):
                 if is_not_empty(ref_value):
                 #if ref_value:
                     p = p.replace('.', '_')
-                    result[p] = render_dict_value(ref_value, full=full, show_key=show_key, gmt=gmt, date_format=date_format, datetime_format=datetime_format, time_format=time_format)
+                    result[p] = render_dict_value(ref_value, full=full, show_key=show_key, gmt=gmt, date_format=date_format, datetime_format=datetime_format, time_format=time_format, deep_level=deep_level)
                                         
         #logger.debug('DictModel: result=%s', result)
         
@@ -302,12 +327,18 @@ class BaseNModel(DictModel, NDBModel):
             if more:
                 return search_results, next_cursor.to_websafe_string().decode('utf-8')  
             else:    
-                return search_results, 'None'
+                #return search_results, 'None'
+                return search_results, None
             
         else:
-            search_results = query.fetch(limit=limit, offset=offset)
+            search_results = query.fetch(limit=limit, offset=offset, keys_only=keys_only)
             
             return search_results
+    
+    @classmethod
+    def delete_multiples(cls, query, limit=model_conf.MAX_FETCH_RECORD):
+        list_of_keys = query.fetch(limit=limit, offset=0, keys_only=True)
+        ndb.delete_multi(list_of_keys)
         
     def before_put(self):
         pass
@@ -324,21 +355,25 @@ class BaseNModel(DictModel, NDBModel):
 class FullTextSearchable(ndb.Model):
     full_text_search_field      = ndb.StringProperty(repeated=True)
     
-    fulltextsearch_field_name   = None
+    fulltextsearch_field_name           = None
+    fulltextsearch_field_to_lowercase   = True
+    
     
     def _pre_put_hook(self):
         """before save, parse searchable field into strings"""
         
-        
-        
         _searchable_field_name = self.fulltextsearch_field_name
         
         logger.debug('_searchable_field_name=%s', _searchable_field_name)
         if _searchable_field_name:
             searchable_field_value = getattr(self, _searchable_field_name)
             logger.debug('searchable_field_value=%s', searchable_field_value)
+            
             if is_not_empty(searchable_field_value):
-                splitted_words = searchable_field_value.lower().split(' ')
+                if self.fulltextsearch_field_to_lowercase:
+                    splitted_words = searchable_field_value.lower().split(' ')
+                else:
+                    splitted_words = searchable_field_value.split(' ')
                 
                 logger.debug('splitted_words=%s', splitted_words)
                 
@@ -347,17 +382,23 @@ class FullTextSearchable(ndb.Model):
                 
                 self.full_text_search_field = searchable_words_list
     
-    
+            else:
+                self.full_text_search_field = []
+                
+            
     
     @classmethod
-    def full_text_count(cls, search_text_list, query=None, limit=model_conf.MAX_FETCH_RECORD_FULL_TEXT_SEARCH_PER_PAGE):
+    def full_text_count(cls, search_text_list, query=None, limit=model_conf.MAX_FETCH_RECORD_FULL_TEXT_SEARCH_PER_PAGE, to_lowercase=True):
         
         logger.debug('full_text_count: search_text_list=%s', search_text_list)
         
         lowercase_words_list = []
         
         if is_not_empty(search_text_list):
-            lowercase_words_list = [x.lower() for x in search_text_list]
+            if to_lowercase:
+                lowercase_words_list = [x.lower() for x in search_text_list]
+            else:
+                lowercase_words_list = search_text_list
         
             logger.debug('full_text_search: lowercase_words_list=%s', lowercase_words_list)
             
@@ -376,7 +417,7 @@ class FullTextSearchable(ndb.Model):
         
     @classmethod
     def full_text_search(cls, text_search = None, query=None, offset=0,  start_cursor=None, return_with_cursor=False, keys_only=False, 
-                         limit=model_conf.MAX_FETCH_RECORD_FULL_TEXT_SEARCH_PER_PAGE):
+                         limit=model_conf.MAX_FETCH_RECORD_FULL_TEXT_SEARCH_PER_PAGE, to_lowercase=True):
         if query is None:
             query = cls.query()
         
@@ -391,7 +432,10 @@ class FullTextSearchable(ndb.Model):
         
             
         if is_not_empty(text_search):
-            lowercase_words_list = [x.lower() for x in text_search]
+            if to_lowercase:
+                lowercase_words_list = [x.lower() for x in text_search]
+            else:
+                lowercase_words_list = text_search
             
             logger.debug('full_text_search: lowercase_words_list=%s', lowercase_words_list)
             
@@ -444,7 +488,25 @@ class FullTextSearchable(ndb.Model):
             
             logger.debug('==========================================')
             
-            return search_results        
+            return search_results
+        
+    @classmethod
+    def full_text_search_query(cls, text_search = None, query=None, to_lowercase=True):
+        if query is None:
+            query = cls.query()
+            
+        if is_not_empty(text_search):
+            if to_lowercase:
+                lowercase_words_list = [x.lower() for x in text_search]
+            else:
+                lowercase_words_list = text_search
+            
+            logger.debug('full_text_search: lowercase_words_list=%s', lowercase_words_list)
+            
+            for word in lowercase_words_list:
+                query = query.filter(ndb.AND(cls.full_text_search_field == word))
+        
+        return query        
             
 
     
