@@ -1,9 +1,7 @@
 
 from google.cloud import ndb
 from trexmodel.models.datastore.ndb_models import BaseNModel, DictModel
-from trexmodel.models.datastore.user_models import User
-from trexmodel.models.datastore.merchant_models import Outlet, MerchantAcct,\
-    ReceiptSetup
+from trexmodel.models.datastore.merchant_models import Outlet
 from trexlib.utils.string_util import random_number, is_not_empty
 import logging, json
 from trexmodel import conf
@@ -19,7 +17,7 @@ class POSSetting(BaseNModel,DictModel):
     '''
     merchant_acct as ancestor
     '''
-    pos_name                = ndb.StringProperty(required=True)
+    device_name             = ndb.StringProperty(required=True)
     activation_code         = ndb.StringProperty(required=True)
     device_id               = ndb.StringProperty(required=False)
     activated               = ndb.BooleanProperty(required=True, default=False)
@@ -27,8 +25,14 @@ class POSSetting(BaseNModel,DictModel):
     #assigned_catalogue      = ndb.KeyProperty(name="assigned_outlet", kind=ProductCatalogue)
     created_datetime        = ndb.DateTimeProperty(required=True, auto_now_add=True)
     activated_datetime      = ndb.DateTimeProperty(required=False)
+    testing                 = ndb.BooleanProperty(required=False, default=False)
     
-    dict_properties = ['pos_name', 'activation_code', 'device_id', 'activated', 'assigned_outlet_key', 'activated_datetime', 'created_datetime']
+    dict_properties = ['device_name', 'activation_code', 'device_id', 'activated', 'assigned_outlet_key', 
+                       'activated_datetime', 'created_datetime']
+    
+    @property
+    def is_test_setting(self):
+        return self.testing
     
     @property
     def assigned_outlet_key(self):
@@ -43,7 +47,7 @@ class POSSetting(BaseNModel,DictModel):
         return self.assigned_outlet_entity.merchant_acct_entity
     
     @staticmethod
-    def create(pos_name, merchant_acct, assign_outlet):
+    def create(device_name, merchant_acct, assign_outlet):
         activation_code = random_number(16)
         checking_activation_pos_setting = POSSetting.get_by_activation_code(activation_code)
         regenerate_activation_code = False
@@ -60,7 +64,7 @@ class POSSetting(BaseNModel,DictModel):
         
         pos_setting = POSSetting(
                                 parent                  = merchant_acct.create_ndb_key(),
-                                pos_name                = pos_name,
+                                device_name             = device_name,
                                 activation_code         = activation_code,
                                 assigned_outlet         = assign_outlet.create_ndb_key(),
                                 
@@ -70,11 +74,11 @@ class POSSetting(BaseNModel,DictModel):
         return pos_setting
     
     @staticmethod
-    def update(pos_setting_key, pos_name, assigned_outlet, device_id):
+    def update(pos_setting_key, device_name, assigned_outlet):
         pos_setting                         = POSSetting.fetch(pos_setting_key)
-        pos_setting.pos_name                = pos_name
+        pos_setting.device_name             = device_name
         pos_setting.assigned_outlet         = assigned_outlet.create_ndb_key()
-        pos_setting.device_id               = device_id
+        
         pos_setting.put()
         
         return pos_setting
@@ -117,7 +121,7 @@ class POSSetting(BaseNModel,DictModel):
         
     def activate(self, device_id):
         self.device_id          = device_id
-        #self.activated          = True
+        self.activated          = True
         self.activated_datetime = datetime.utcnow()
         self.put()
         
@@ -204,7 +208,7 @@ class POSSetting(BaseNModel,DictModel):
         
         return {
                 'activation_code'                   : self.activation_code,
-                'pos_name'                          : self.pos_name,
+                'device_name'                       : self.device_name,
                 'company_name'                      : merchant_acct.company_name,
                 'website'                           : merchant_acct.website,
                 'account_id'                        : self.parent_key,
@@ -309,29 +313,32 @@ class DinningTableSetup(BaseNModel,DictModel):
     table_list              = ndb.StringProperty(required=True)
     assigned_outlet_list    = ndb.JsonProperty()
     is_publish              = ndb.BooleanProperty(default=False)
+    show_occupied           = ndb.BooleanProperty(default=False)
     modified_datetime       = ndb.DateTimeProperty(auto_now=True)
     
     
-    dict_properties = ['name', 'table_list', 'assigned_outlet_list', 'is_publish']
+    dict_properties = ['name', 'table_list', 'assigned_outlet_list', 'is_publish', 'show_occupied']
     
     @staticmethod
-    def create(name, table_list, assigned_outlet_key_list, merchant_acct):
+    def create(name, table_list, assigned_outlet_key_list, merchant_acct, show_occupied=False):
         dinning_table_setup = DinningTableSetup(
                                         parent                  = merchant_acct.create_ndb_key(),
                                         name                    = name, 
                                         is_publish              = False,
                                         table_list              = table_list,
+                                        show_occupied           = show_occupied,
                                         assigned_outlet_list    = assigned_outlet_key_list,
                                         )
         dinning_table_setup.put()
         return dinning_table_setup
     
     @staticmethod
-    def update(dinning_table_setup, name, table_list, assigned_outlet_key_list):
+    def update(dinning_table_setup, name, table_list, assigned_outlet_key_list, show_occupied=False):
         dinning_table_setup.name                    = name
         dinning_table_setup.table_list              = table_list
         dinning_table_setup.assigned_outlet_list    = assigned_outlet_key_list
         dinning_table_setup.is_publish              = False
+        dinning_table_setup.show_occupied           = show_occupied
         dinning_table_setup.put()
     
     @staticmethod    
@@ -344,6 +351,7 @@ class DinningTableSetup(BaseNModel,DictModel):
           
         for outlet_key in self.assigned_outlet_list:
             Outlet.assign_dinning_table_setup_to_outlet(outlet_key, self.table_list)
+            Outlet.assign_dinning_table_control_to_outlet(outlet_key, self.show_occupied)
         
         self.put()
             
@@ -353,6 +361,7 @@ class DinningTableSetup(BaseNModel,DictModel):
           
         for outlet_key in self.assigned_outlet_list:
             Outlet.remove_dinning_table_setup_from_outlet(outlet_key)
+            Outlet.assign_dinning_table_control_to_outlet(outlet_key, False)
             
         self.put()         
         
