@@ -9,11 +9,14 @@ from trexmodel.models.datastore.user_models import User
 from trexmodel.models.datastore.merchant_models import MerchantAcct, Outlet, MerchantUser
 import logging, json
 from trexmodel import conf, program_conf
-from trexlib.utils.string_util import random_string
+from trexlib.utils.string_util import random_string, random_number, is_not_empty
 from datetime import datetime, timedelta
 import trexmodel.conf as model_conf
 from trexlib.utils.common.common_util import sort_dict_list
 from trexmodel.utils.model.model_util import generate_transaction_id
+from dateutil.relativedelta import relativedelta
+from trexmodel.program_conf import PRODUCT_TYPES, LOYALTY_PRODUCT
+from trexlib.utils.crypto_util import encrypt
 
 logger = logging.getLogger('model')
 
@@ -95,6 +98,8 @@ class PrepaidSettings(BaseNModel,DictModel):
         if self.is_multi_tier_prepaid:
             prepaid_program_configuration['multitier_settings'] = self.multitier_settings
         
+        logger.debug('prepaid_program_configuration=%s', prepaid_program_configuration)
+        
         return prepaid_program_configuration
     
     @staticmethod
@@ -106,10 +111,12 @@ class PrepaidSettings(BaseNModel,DictModel):
                 not_archive_program_list.append(r)
         return not_archive_program_list
     
-    def __update_merchant_account_prepaid_configuration(self):
+    def update_merchant_account_prepaid_configuration(self):
         merchant_acct = self.merchant_acct_entity    
         merchant_acct.update_prepaid_program(self.to_configuration())
         merchant_acct.put()
+        
+        
     
     @staticmethod
     def create(merchant_acct, label, start_date, end_date, is_multi_tier_prepaid=False, is_lump_sum_prepaid=True, lump_sum_settings=None, multitier_settings=None, created_by=None):
@@ -131,7 +138,7 @@ class PrepaidSettings(BaseNModel,DictModel):
         prepaid_settings.put()
         
         if prepaid_settings.enable:
-            prepaid_settings.__update_merchant_account_prepaid_configuration()
+            prepaid_settings.update_merchant_account_prepaid_configuration()
         
         return prepaid_settings
         
@@ -150,7 +157,7 @@ class PrepaidSettings(BaseNModel,DictModel):
         prepaid_settings.put()    
     
         if prepaid_settings.enable:
-            prepaid_settings.__update_merchant_account_prepaid_configuration()
+            prepaid_settings.update_merchant_account_prepaid_configuration()
     
     
         return prepaid_settings
@@ -160,7 +167,7 @@ class PrepaidSettings(BaseNModel,DictModel):
         prepaid_settings.enabled = True
         prepaid_settings.put()
         
-        prepaid_settings.__update_merchant_account_prepaid_configuration()
+        prepaid_settings.update_merchant_account_prepaid_configuration()
         
     @staticmethod
     def disable(prepaid_settings):
@@ -180,6 +187,160 @@ class PrepaidSettings(BaseNModel,DictModel):
         merchant_acct.remove_prepaid_program_configuration(prepaid_settings.key_in_str)
         merchant_acct.put()    
 
+class PrepaidRedeemSettings(BaseNModel,DictModel):
+    '''
+    Merchant acct as ancestor
+    '''
+    label                               = ndb.StringProperty(required=True)
+    device_activation_code              = ndb.StringProperty(required=True)
+    device_type                         = ndb.StringProperty(required=False, default=LOYALTY_PRODUCT, choices=set(PRODUCT_TYPES))
+    assigned_outlet                     = ndb.KeyProperty(name="assigned_outlet", kind=Outlet)
+    testing                             = ndb.BooleanProperty(required=False, default=False)
+    
+    redeem_code                         = ndb.StringProperty(required=True)
+    
+    created_datetime                    = ndb.DateTimeProperty(required=True, auto_now_add=True)
+    modified_datetime                   = ndb.DateTimeProperty(required=True, auto_now=True)
+    
+    created_by                          = ndb.KeyProperty(name="created_by", kind=MerchantUser)
+    created_by_username                 = ndb.StringProperty(required=False)
+    modified_by                         = ndb.KeyProperty(name="modified_by", kind=MerchantUser)
+    modified_by_username                = ndb.StringProperty(required=False)
+    
+    
+    dict_properties = [
+                        'label', 'redeem_code', 'redeem_url', 'device_activation_code', 'device_type', 'assigned_outlet_key', 'created_datetime', 'modified_datetime',
+                    ]
+    
+    @property
+    def is_test_setting(self):
+        return self.testing
+    
+    @property
+    def assigned_outlet_key(self):
+        return self.assigned_outlet.urlsafe().decode('utf-8')
+    
+    @property
+    def assigned_outlet_entity(self):
+        return Outlet.fetch(self.assigned_outlet_key)
+    
+    @property
+    def merchant_acct_entity(self):
+        return MerchantAcct.fetch(self.key.parent().urlsafe())
+    
+    @property
+    def redeem_url(self):
+        encrypted_redeem_code = encrypt(self.redeem_code)
+        return conf.PREPAID_REDEEM_URL.format(code=encrypted_redeem_code)
+    
+    @property
+    def is_loyalty_device(self):
+        if self.device_type == program_conf.LOYALTY_PRODUCT:
+            return True
+        return False
+    
+    @staticmethod
+    def create(merchant_acct, label, assigned_outlet, device_activation_code=None, testing=False, 
+               created_by=None
+               ):
+        
+        created_by_username = None
+        if is_not_empty(created_by):
+            if isinstance(created_by, MerchantUser):
+                created_by_username = created_by.username
+        
+        redeem_code = random_number(24)
+        checking_redeem_setting = PrepaidRedeemSettings.get_by_redeem_code(redeem_code)
+        regenerate_redeem_code = False
+        
+        if checking_redeem_setting:
+            regenerate_redeem_code = True
+        
+        while(regenerate_redeem_code):
+            redeem_code = random_number(24)
+            checking_redeem_setting = PrepaidRedeemSettings.get_by_redeem_code(redeem_code)
+            if checking_redeem_setting==None:
+                regenerate_redeem_code = False
+                
+        prepaid_redeem_settings = PrepaidRedeemSettings(
+                                        parent                  = merchant_acct.create_ndb_key(),
+                                        assigned_outlet         = assigned_outlet.create_ndb_key(),
+                                        label                   = label,
+                                        redeem_code             = redeem_code,
+                                        testing                 = testing, 
+                                        device_activation_code  = device_activation_code,
+                                        
+                                        created_by              = created_by.create_ndb_key(),
+                                        created_by_username     = created_by_username,   
+                                    )
+        
+        prepaid_redeem_settings.put()
+        
+        return prepaid_redeem_settings
+        
+    @staticmethod
+    def update(prepaid_redeem_settings_key, label, assigned_outlet, device_activation_code=None, 
+               modified_by=None
+               ):
+        
+        modified_by_username = None
+        if is_not_empty(modified_by):
+            if isinstance(modified_by, MerchantUser):
+                modified_by_username = modified_by.username
+        
+        prepaid_redeem_settings = PrepaidRedeemSettings.fetch(prepaid_redeem_settings_key)
+        
+        prepaid_redeem_settings.label                   = label
+        prepaid_redeem_settings.device_activation_code  = device_activation_code
+        prepaid_redeem_settings.assigned_outlet         = assigned_outlet.create_ndb_key()
+        prepaid_redeem_settings.modified_by             = modified_by.create_ndb_key()
+        prepaid_redeem_settings.modified_by_username    = modified_by_username
+        
+        prepaid_redeem_settings.put()
+        
+        return prepaid_redeem_settings    
+    
+    @staticmethod
+    def remove(prepaid_redeem_settings_key):
+        prepaid_redeem_settings = PrepaidRedeemSettings.fetch(prepaid_redeem_settings_key)
+        if prepaid_redeem_settings:
+            prepaid_redeem_settings.delete()
+    
+    @staticmethod
+    def get_by_redeem_code(redeem_code):
+        return PrepaidRedeemSettings.query(PrepaidRedeemSettings.redeem_code ==redeem_code).get()
+    
+    @staticmethod
+    def list_by_merchant_account_and_assigned_outlet(merchant_acct, assigned_outlet, offset=0, limit=conf.PAGINATION_SIZE, start_cursor=None, return_with_cursor=False):
+        query = PrepaidRedeemSettings.query(ndb.AND(
+                        PrepaidRedeemSettings.assigned_outlet==assigned_outlet.create_ndb_key()
+                        ),ancestor=merchant_acct.create_ndb_key())
+        
+        return PrepaidRedeemSettings.list_all_with_condition_query(query, offset=offset, limit=limit, start_cursor=start_cursor, return_with_cursor=return_with_cursor)
+    
+    @staticmethod
+    def count_by_merchant_account_and_assigned_outlet(merchant_acct, assigned_outlet):
+        query = PrepaidRedeemSettings.query(ndb.AND(
+                        PrepaidRedeemSettings.assigned_outlet==assigned_outlet.create_ndb_key()
+                        ),ancestor=merchant_acct.create_ndb_key())
+        
+        return PrepaidRedeemSettings.count_with_condition_query(query)
+    
+    @staticmethod
+    def list_by_merchant_account(merchant_acct, offset=0, limit=conf.PAGINATION_SIZE, start_cursor=None, return_with_cursor=False):
+        query = PrepaidRedeemSettings.query(ancestor=merchant_acct.create_ndb_key())
+        
+        return PrepaidRedeemSettings.list_all_with_condition_query(query, offset=offset, limit=limit, start_cursor=start_cursor, return_with_cursor=return_with_cursor)
+    
+    @staticmethod
+    def count_by_merchant_acct(merchant_acct):
+        if merchant_acct:
+            query = PrepaidRedeemSettings.query(ancestor=merchant_acct.create_ndb_key())
+        else:
+            query = PrepaidRedeemSettings.query()
+        
+        return PrepaidRedeemSettings.count_with_condition_query(query)
+    
 class CustomerPrepaidReward(BaseNModel,DictModel):
     '''
     Customer acct as ancestor
@@ -203,12 +364,19 @@ class CustomerPrepaidReward(BaseNModel,DictModel):
     prepaid_scheme_details              = ndb.JsonProperty(required=False)
     
     topup_datetime                      = ndb.DateTimeProperty(required=True, auto_now_add=True)
-    topup_by                            = ndb.KeyProperty(name="created_by", kind=MerchantUser)
+    topup_by                            = ndb.KeyProperty(name="topup_by", kind=MerchantUser)
+    topup_by_username                   = ndb.StringProperty(required=False)
     
     reverted_datetime                   = ndb.DateTimeProperty(required=False)
     reverted_by                         = ndb.KeyProperty(name="reverted_by", kind=MerchantUser)
     reverted_by_username                = ndb.StringProperty(required=False)
     
+    dict_properties         = ['transaction_id', 'invoice_id', 'topup_amount', 'topup_unit', 'topup_prepaid_rate',
+                               'prepaid_amount', 'used_prepaid_amount', 'status', 'prepaid_scheme_details',
+                               'topup_datetime', 'topup_by', 'reverted_datetime', 'reverted_by_username',
+                               'status'
+                               ]
+    
     @property
     def is_valid(self):
         return self.status == program_conf.REWARD_STATUS_VALID
@@ -217,21 +385,50 @@ class CustomerPrepaidReward(BaseNModel,DictModel):
     def is_redeemed(self):
         return self.status == program_conf.REWARD_STATUS_REDEEMED
     
+    @property
+    def is_used(self):
+        return self.used_prepaid_amount>0
+    
     @property
     def prepaid_balance(self):
         return self.prepaid_amount - self.used_prepaid_amount
     
+    @property
+    def expiry_date(self):
+        return (self.topup_datetime + relativedelta(years=100)).date()
+    
+    @property
+    def reward_format_key(self):
+        return None
+    
+    @property
+    def rewarded_datetime(self):
+        return self.topup_datetime
+    
+    @property
+    def reward_amount(self):
+        return self.prepaid_amount
+    
     def update_used_reward_amount(self, used_prepaid_amount):
         self.used_prepaid_amount    += used_prepaid_amount
         prepaid_balance              = self.prepaid_balance
         
+        if prepaid_balance<0:
+            prepaid_balance = 0
+        
         logger.debug('CustomerCountableReward: prepaid_balance=%s', prepaid_balance)
         
         if prepaid_balance ==0:
             self.status = program_conf.REWARD_STATUS_REDEEMED
-        
+        else:
+            self.status = program_conf.REWARD_STATUS_VALID
+            
         self.put()
     
+    @property
+    def reward_format(self):
+        return program_conf.REWARD_FORMAT_PREPAID
+    
     @staticmethod
     def __calculate_topup_unit(topup_amount, prepaid_scheme_details):
         return int(topup_amount/prepaid_scheme_details.get('topup_amount'))
@@ -246,19 +443,48 @@ class CustomerPrepaidReward(BaseNModel,DictModel):
     
     def to_prepaid_summary(self):
         prepaid_summary =  {
-                            'amount'            : self.prepaid_amount,
-                            'used_amount'       : self.used_prepaid_amount,    
+                            'amount'                : self.prepaid_amount,
+                            'used_amount'           : self.used_prepaid_amount,  
+                               
                             }
         
         return prepaid_summary
     
+    def to_reward_summary(self):
+        return self.to_prepaid_summary()
+    
+    @property
+    def is_reach_reward_limit(self):
+        return self.status == program_conf.REWARD_STATUS_REACH_LIMIT
+    
+    @property
+    def reward_balance(self):
+        return self.prepaid_amount - self.used_prepaid_amount
+    
+    @property
+    def reward_format_label(self):
+        return 'prepaid'
+    
+    @property
+    def reward_brief(self):
+        return 'Entitle {reward_amount} {reward_format}'.format(reward_amount=self.prepaid_amount, reward_format=self.reward_format_label)
+    
+    @classmethod
+    def list_by_valid_with_cursor(cls, customer, limit=50, start_cursor=None):
+        query = cls.query(ndb.AND(cls.status==program_conf.REWARD_STATUS_VALID
+            ), ancestor=customer.create_ndb_key()).order(cls.topup_datetime)
+            
+        (result, next_cursor) = cls.list_all_with_condition_query(query, start_cursor=start_cursor, return_with_cursor=True, limit=limit)
+        
+        return (result, next_cursor) 
+    
     @staticmethod
     def list_by_customer(customer, status=program_conf.REWARD_STATUS_VALID, limit = conf.MAX_FETCH_RECORD):
         return CustomerPrepaidReward.query(ndb.AND(CustomerPrepaidReward.status==status), ancestor=customer.create_ndb_key()).fetch(limit=limit)
     
     @staticmethod
-    def list_all_by_customer(customer, limit = conf.MAX_FETCH_RECORD):
-        return CustomerPrepaidReward.query(ancestor=customer.create_ndb_key()).fetch(limit=limit)
+    def list_all_by_customer(customer, limit = conf.MAX_FETCH_RECORD, offset=0):
+        return CustomerPrepaidReward.query(ancestor=customer.create_ndb_key()).fetch(offset=offset, limit=limit)
     
     @staticmethod
     def list_by_transaction_id(transaction_id):
@@ -269,56 +495,71 @@ class CustomerPrepaidReward(BaseNModel,DictModel):
         return CustomerPrepaidReward.query(ndb.AND(CustomerPrepaidReward.status==program_conf.REDEEM_STATUS_VALID),ancestor=customer_acct.create_ndb_key()).fetch(limit=conf.MAX_FETCH_RECORD)
         
     @staticmethod
-    def topup(customer_acct, topup_amount, prepaid_program, invoice_id=None, topup_outlet=None, topup_by=None, transaction_id=None):
+    def topup(customer_acct, topup_amount, prepaid_program, invoice_id=None, topup_outlet=None, topup_by=None, transaction_id=None, topup_datetime=None):
+        prepaid_scheme_details      = {
+                                        }
+        found_match_scheme          = None
         
-        is_lump_sum_prepaid     = prepaid_program.is_lump_sum_prepaid
-        is_multi_tier_prepaid   = prepaid_program.is_multi_tier_prepaid
-        
-        sorted_tier_scheme      = None
-        
-        prepaid_scheme_details  = {
-                                    'program_key'   : prepaid_program.key_in_str
-                                    }
-        
-        prepaid_scheme      = None
-        
-        if is_multi_tier_prepaid:
-            sorted_tier_scheme = sort_dict_list(prepaid_program.multitier_settings.values(), 'min_topup_amount')
+        if prepaid_program:
+            is_lump_sum_prepaid     = prepaid_program.is_lump_sum_prepaid
+            is_multi_tier_prepaid   = prepaid_program.is_multi_tier_prepaid
             
-            logger.debug('Get topup scheme from topup amount(%s)', topup_amount)
+            sorted_tier_scheme      = None
+            
+            prepaid_scheme_details  = {
+                                        'program_key'   : prepaid_program.key_in_str
+                                        }
+            
+            tier_prepaid_scheme      = None
+            
+            
+            logger.debug('is_multi_tier_prepaid=%s', is_multi_tier_prepaid)
+            
+            if is_multi_tier_prepaid:
+                sorted_tier_scheme = sort_dict_list(prepaid_program.multitier_settings.values(), 'min_topup_amount')
+                
+                logger.debug('Get topup scheme from topup amount(%s)', topup_amount)
+                
+                #look for higheest min topup amount scheme
+                for scheme in sorted_tier_scheme:
+                    if topup_amount>= scheme.get('min_topup_amount'):
+                        tier_prepaid_scheme = {
+                                                'topup_amount'  : scheme.get('topup_amount'),
+                                                'prepaid_amount': scheme.get('prepaid_amount'),
+                                                'prepaid_rate'  : scheme.get('prepaid_amount')/scheme.get('topup_amount'),
+                                                'scheme_type'   : 'tier',
+                                                }
+                        logger.debug('Found topup scheme for min topup amount(%s)', scheme.get('min_topup_amount'))
+            
+            logger.debug('tier_prepaid_scheme=%s', tier_prepaid_scheme)
+            
+            #if topup amount is less than smallest prepaid tier minimum topup amount, thus goto lump sum scheme    
+            if tier_prepaid_scheme is None and is_lump_sum_prepaid:
+                logger.debug('going to create scheme from lump sum setting')
+                prepaid_scheme = prepaid_program.lump_sum_settings
+                found_match_scheme = {
+                                                'topup_amount'  : prepaid_scheme.get('topup_amount'),
+                                                'prepaid_amount': prepaid_scheme.get('prepaid_amount'),
+                                                'prepaid_rate'  : prepaid_scheme.get('prepaid_amount')/prepaid_scheme.get('topup_amount'),
+                                                'scheme_type'   : 'lump_sum',
+                                                }  
+            
+            else:
+                found_match_scheme = tier_prepaid_scheme
+             
+        if found_match_scheme is None:
+            logger.debug('going to create scheme default scheme')
             
-            for scheme in sorted_tier_scheme:
-                if topup_amount>= scheme.get('min_topup_amount'):
-                    prepaid_scheme = {
-                                            'topup_amount'  : scheme.get('topup_amount'),
-                                            'prepaid_amount': scheme.get('prepaid_amount'),
-                                            'prepaid_rate'  : scheme.get('prepaid_amount')/scheme.get('topup_amount'),
-                                            'scheme_type'   : 'tier',
-                                            }
-                    logger.debug('Found topup scheme for min topup amount(%s)', scheme.get('min_topup_amount'))
-        
-        logger.debug('prepaid_scheme=%s', prepaid_scheme)
-        
-        #if topup amount is less than smallest prepaid tier minimum topup amount, thus goto lump sum scheme    
-        if prepaid_scheme is None and is_lump_sum_prepaid:
             found_match_scheme = {
-                                            'topup_amount'  : scheme.get('topup_amount'),
-                                            'prepaid_amount': scheme.get('prepaid_amount'),
-                                            'prepaid_rate'  : scheme.get('prepaid_amount')/scheme.get('topup_amount'),
-                                            'scheme_type'   : 'lump_sum',
-                                            }  
-        
-        if prepaid_scheme is None:
-            prepaid_scheme = {
                                             'topup_amount'  : 1,
                                             'prepaid_amount': 1,
                                             'prepaid_rate'  : 1,
                                             'scheme_type'   : 'auto',
                                             }
             
-            logger.debug('create default prepaid_scheme=%s', prepaid_scheme)
+        logger.debug('create found_match_scheme=%s', found_match_scheme)
         
-        prepaid_scheme_details.update(prepaid_scheme)
+        prepaid_scheme_details.update(found_match_scheme)
         
         logger.debug('prepaid_scheme_details=%s', prepaid_scheme_details)
             
@@ -331,11 +572,14 @@ class CustomerPrepaidReward(BaseNModel,DictModel):
         if topup_by:
             topup_by_key = topup_by.create_ndb_key()
             
+        if topup_datetime is None:
+            topup_datetime = datetime.utcnow()
+            
         merchant_acct               = customer_acct.registered_merchant_acct
         prepaid_topup_reward        = CustomerPrepaidReward(
                                                             parent                  = customer_acct.create_ndb_key(),
                                                             merchant_acct           = merchant_acct.create_ndb_key(),
-                                                            topup_outlet            = topup_outlet.create_ndb_key(),
+                                                            topup_outlet            = topup_outlet.create_ndb_key() if topup_outlet else None,
                                                             topup_amount            = topup_amount,
                                                             topup_unit              = topup_unit,
                                                             prepaid_amount          = prepaid_amount,
@@ -347,6 +591,8 @@ class CustomerPrepaidReward(BaseNModel,DictModel):
                                                             invoice_id              = invoice_id,
                                                             
                                                             topup_by                = topup_by_key,
+                                                            
+                                                            topup_datetime          = topup_datetime,    
                                                             )
         
         prepaid_topup_reward.put()
@@ -354,3 +600,8 @@ class CustomerPrepaidReward(BaseNModel,DictModel):
         return prepaid_topup_reward
     
     
+    @staticmethod
+    def delete_all_by_customer(customer):
+        query = CustomerPrepaidReward.query(ancestor=customer.create_ndb_key())
+        CustomerPrepaidReward.delete_multiples(query)
+    
