@@ -413,6 +413,10 @@ class Product(ProductBase, FullTextSearchable):
     
     dict_properties = ['product_sku', 'product_name', 'category_key', 'price', 'cost', 'product_desc', 'barcode', 'pos_settings', 'tax_details', 'product_modifier']
     
+    @property
+    def product_default_image(self):
+        return self.pos_settings.get('representation_settings').get('image_url')
+    
     def setting_details_for_pos(self, merchant_acct):
         modifier_keys_list              = self.product_modifier
         modifier_setting_list_for_pos   = []
@@ -758,6 +762,8 @@ class ProductCatalogue(ProductBase):
     menu_settings               = ndb.JsonProperty()  
     published_menu_settings     = ndb.JsonProperty()
     
+    
+    
     is_publish                  = ndb.BooleanProperty(default=False)
     
     created_datetime            = ndb.DateTimeProperty(required=True, auto_now_add=True)
