@@ -11,7 +11,6 @@ from trexmodel.models.datastore.merchant_models import MerchantAcct, Outlet,\
     MerchantUser
 from trexmodel.models.datastore.coporate_models import CorporateAcct
 import logging
-from trexlib.utils.common.cache_util import cache
 from trexmodel import conf, program_conf
 from datetime import datetime
 from trexmodel.models.datastore.model_decorators import model_transactional
@@ -46,6 +45,8 @@ class BaseProgram(BaseNModel, DictModel):
     enabled                             = ndb.BooleanProperty(default=True)
     archived                            = ndb.BooleanProperty(default=False)
     
+    loyalty_package                     = ndb.StringProperty(required=False, default=program_conf.LOYALTY_PACKAGE_SCALE)
+    
     @property
     def is_enabled(self):
         return self.enabled
@@ -106,12 +107,18 @@ class BaseProgram(BaseNModel, DictModel):
     @property
     def is_recurring_scheme(self):
         if self.program_settings and self.program_settings.get('scheme'):
-            return self.program_settings.get('scheme').get('is_recurring_scheme') or True
+            return self.program_settings.get('scheme').get('is_recurring_scheme')
+        return False
+        
+    @property
+    def giveaway_system_condition(self):
+        if self.program_settings and self.program_settings.get('giveaway_system_settings'):
+            return self.program_settings.get('giveaway_system_settings').get('giveaway_system_condition')
         
     @property
     def expiration_type(self):
         if self.program_settings and self.program_settings.get('scheme'):
-            return self.program_settings.get('scheme').get('expiration_type')
+            return self.program_settings.get('scheme').get('expiration_type')    
         
     @property
     def is_expiration_date_type(self):
@@ -153,7 +160,7 @@ class BaseProgram(BaseNModel, DictModel):
             
         program.modified_by            = modified_by.create_ndb_key()
         program.modified_by_username   = modified_by_username
-        program.completed_status       = program_conf.PROGRAM_STATUS_REWARD_EXCLUSIVITY
+        program.completed_status       = program_conf.PROGRAM_STATUS_EXCLUSIVITY
         program.program_settings       = program_settings
         
         program.put()
@@ -163,11 +170,18 @@ class BaseProgram(BaseNModel, DictModel):
     @classmethod
     def __create_schedule_program(cls, merchant_acct, program):
         if program.reward_base in (program_conf.SCHEDULE_BASED_PROGRAM):
+            
             if program.reward_base == program_conf.REWARD_BASE_ON_BIRTHDAY:
                 if program.giveaway_reward_when == program_conf.PROGRAM_SCHEDULE_TYPE_MONTH_START:
                     MerchantScheduleProgram.create_first_day_of_month_schedule_program(merchant_acct, program)
+                
                 elif program.giveaway_reward_when == program_conf.ADVANCE_IN_DAY:
                     MerchantScheduleProgram.create_daily_schedule_program(merchant_acct, program)
+                    
+            elif program.reward_base == program_conf.REWARD_BASE_ON_GIVEAWAY:
+                if program.giveaway_system_condition == program_conf.GIVEAWAY_SYSTEM_CONDITION_MEMBERSHIP_YEAR:
+                    MerchantScheduleProgram.create_daily_schedule_program(merchant_acct, program)
+                        
     
     @classmethod
     def __remove_schedule_program(cls, program):
@@ -182,7 +196,7 @@ class BaseProgram(BaseNModel, DictModel):
         program.put()
         
         merchant_acct = program.merchant_acct
-        merchant_acct.update_published_program(program.to_program_configuration())
+        merchant_acct.update_published_program(program.to_configuration())
         
         cls.__create_schedule_program(merchant_acct, program)
         
@@ -217,7 +231,7 @@ class BaseProgram(BaseNModel, DictModel):
         
         if program.is_published:
             merchant_acct = program.merchant_acct
-            merchant_acct.update_published_program(program.to_program_configuration())
+            merchant_acct.update_published_program(program.to_configuration())
             
             cls.__create_schedule_program(merchant_acct, program)
             
@@ -243,10 +257,11 @@ class BaseProgram(BaseNModel, DictModel):
         
         cls.__remove_schedule_program(program)
         
-    def to_program_configuration(self):
+    def to_configuration(self):
         program_configuration = {
                                 'merchant_acct_key'                 : self.parent_key,
                                 'program_key'                       : self.key_in_str,
+                                'label'                             : self.label,
                                 'desc'                              : self.desc,
                                 'reward_base'                       : self.reward_base,
                                 'reward_format'                     : self.reward_format,
@@ -265,7 +280,8 @@ class BasicRewardProgram(BaseProgram):
     
     @property
     def completed_prograss_in_percentage(self):
-        return program_conf.program_completed_progress_percentage(self.completed_status)
+        
+        return program_conf.program_completed_progress_percentage(self.completed_status, self.loyalty_package)
     
     @property
     def completed_status_index(self):
@@ -273,8 +289,9 @@ class BasicRewardProgram(BaseProgram):
     
         
     @classmethod
-    def update_program_base_data(cls, program, reward_base=None, reward_format=None,  
-                                 desc=None,start_date=None, end_date=None, modified_by=None):
+    def update_program_base_data(cls, program, label=None, reward_base=None, reward_format=None,  
+                                 desc=None,start_date=None, end_date=None, modified_by=None, 
+                                 loyalty_package=program_conf.LOYALTY_PACKAGE_LITE):
         
         modified_by_username = None
         
@@ -291,17 +308,17 @@ class BasicRewardProgram(BaseProgram):
                                                     }
         
         program_settings['exclusivity'] = exclusivity_configuration
-        
-        program.reward_base            = reward_base
-        program.reward_format          = reward_format
-        program.desc                   = desc
-        program.start_date             = start_date
-        program.end_date               = end_date
-        program.modified_by            = modified_by.create_ndb_key()
-        program.modified_by_username   = modified_by_username
-        program.completed_status       = program_conf.PROGRAM_STATUS_PROGRAM_BASE 
-        program.program_settings       = program_settings
-        
+        program.label                   = label
+        program.reward_base             = reward_base
+        program.reward_format           = reward_format
+        program.desc                    = desc
+        program.start_date              = start_date
+        program.end_date                = end_date
+        program.modified_by             = modified_by.create_ndb_key()
+        program.modified_by_username    = modified_by_username
+        program.completed_status        = program_conf.PROGRAM_STATUS_PROGRAM_BASE 
+        program.program_settings        = program_settings
+        program.loyalty_package         = loyalty_package
         
         
         program.put()
@@ -314,6 +331,7 @@ class BasicRewardProgram(BaseProgram):
                                             giveaway_system_condition = None, 
                                             giveaway_system_condition_membership_key=None, 
                                             giveaway_system_condition_tier_membership_key=None,
+                                            giveaway_system_condition_value=None,
                                             reward_scheme_configuration={}, modified_by=None,
                                             remarks=None):
         modified_by_username = None
@@ -337,10 +355,12 @@ class BasicRewardProgram(BaseProgram):
             
             if giveaway_system_settings is None:        
                 giveaway_system_settings = {
-                                                    'giveaway_system_condition': giveaway_system_condition
+                                                    'giveaway_system_condition'         : giveaway_system_condition,
+                                                    'giveaway_system_condition_value'   : giveaway_system_condition_value,
                                                 }
             else:
-                giveaway_system_settings['giveaway_system_condition'] = giveaway_system_condition
+                giveaway_system_settings['giveaway_system_condition']       = giveaway_system_condition
+                giveaway_system_settings['giveaway_system_condition_value'] = giveaway_system_condition_value
                 
             if is_not_empty(giveaway_system_condition_membership_key):
                 giveaway_system_settings['giveaway_memberships'] = giveaway_system_condition_membership_key
@@ -357,13 +377,22 @@ class BasicRewardProgram(BaseProgram):
             logger.debug('######################## giveaway_system_settings=%s', giveaway_system_settings)
         
             program_settings['giveaway_system_settings'] = giveaway_system_settings    
-                
+        else:
+            if program_settings.get('giveaway_system_settings'):
+                del program_settings['giveaway_system_settings']
+            
+        if program_conf.REWARD_FORMAT_VOUCHER != program.reward_format:
+            if program_settings.get('reward_items'):
+                del program_settings['reward_items']
+                        
         program.modified_by             = modified_by.create_ndb_key()
         program.modified_by_username    = modified_by_username
         program.completed_status        = program_conf.PROGRAM_STATUS_REWARD_SCHEME
         program.program_settings        = program_settings
         program.remarks                 = remarks
         
+        logger.debug('program_settings=%s', program_settings)
+        
         program.put()
         
         return program
@@ -529,7 +558,7 @@ class BasicRewardProgram(BaseProgram):
     
     
 class MerchantProgram(BasicRewardProgram):
-    dict_properties                     = ['reward_base', 'giveaway_method', 'reward_format', 'completed_status', 'start_date', 'end_date', 'desc', 'program_settings', 
+    dict_properties                     = ['reward_base', 'giveaway_method', 'reward_format', 'completed_status', 'start_date', 'end_date', 'label','desc', 'program_settings', 
                                            'created_datetime', 'modified_datetime',  'enabled','completed_status','is_enabled', 'is_disabled', 'is_review_state', 
                                            'is_published', 'archived', 'is_reward_amount_required', 'completed_prograss_in_percentage', 'completed_status_index',
                                            'exclusive_tags_list', 'exclusive_memberships_list', 'exclusive_tier_memberships_list',
@@ -537,17 +566,24 @@ class MerchantProgram(BasicRewardProgram):
                                            'expiration_type', 'expiration_date', 'expiration_value', 'is_expiration_date_type',
                                            'giveaway_reward_when', 'giveaway_reward_advance_in_day',
                                            'giveaway_system_condition', 'giveaway_system_condition_memberships_list', 'giveaway_system_condition_tier_memberships_list',
+                                           'remarks', 'loyalty_package',
                                            'created_by_username', 'modified_by_username']
     
-    @property
-    def giveaway_system_condition(self):
-        if self.program_settings and self.program_settings.get('giveaway_system_settings'):
-            return self.program_settings.get('giveaway_system_settings').get('giveaway_system_condition')
-    
     @property
     def giveaway_system_condition_memberships_list(self):
         if self.program_settings and self.program_settings.get('giveaway_system_settings'):
-            return self.program_settings.get('giveaway_system_settings').get('giveaway_memberships')
+            
+            giveaway_memberships = self.program_settings.get('giveaway_system_settings').get('giveaway_memberships')
+            if is_not_empty(giveaway_memberships):
+                if isinstance(giveaway_memberships, str):
+                    return giveaway_memberships.split(',')
+                else:
+                    #check the value is value separated by comma
+                    memberships_list = []
+                    for m in giveaway_memberships:
+                        memberships_list.extend(m.split(','))
+                    return memberships_list
+                    
     
     @property
     def giveaway_system_condition_tier_memberships_list(self):
@@ -555,8 +591,10 @@ class MerchantProgram(BasicRewardProgram):
             return self.program_settings.get('giveaway_system_settings').get('giveaway_tier_memberships')
     
     @staticmethod
-    def create(merchant_acct, reward_base=None, reward_format=None, 
-               desc=None, start_date=None, end_date=None, created_by=None):
+    def create(merchant_acct, label=None, reward_base=None, reward_format=None, 
+               desc=None, start_date=None, end_date=None, created_by=None,
+               loyalty_package = program_conf.LOYALTY_PACKAGE_SCALE,
+               ):
         
         created_by_username = None
         if is_not_empty(created_by):
@@ -570,13 +608,17 @@ class MerchantProgram(BasicRewardProgram):
                                                     }
         
         program_settings = {
-                            'exclusivity'               : exclusivity_configuration
+                            'exclusivity'               : exclusivity_configuration,
+                            'scheme'                    : {
+                                                            'is_recurring_scheme': True,
+                                                        }
                             }
         
         
         
         merchant_program =  MerchantProgram(
                                         parent                              = merchant_acct.create_ndb_key(),
+                                        label                               = label,
                                         reward_base                         = reward_base,
                                         reward_format                       = reward_format,
                                         desc                                = desc,
@@ -586,6 +628,7 @@ class MerchantProgram(BasicRewardProgram):
                                         created_by_username                 = created_by_username,
                                         completed_status                    = program_conf.PROGRAM_STATUS_PROGRAM_BASE,
                                         program_settings                    = program_settings,
+                                        loyalty_package                     = loyalty_package,
                                         )
         
         merchant_program.put()
@@ -621,7 +664,7 @@ class MerchantTierRewardProgram(MerchantProgram):
                                            'is_published', 'archived', 'completed_prograss_in_percentage', 'completed_status_index',
                                            'exclusive_tags_list', 'exclusive_memberships_list', 'exclusive_tier_memberships_list',
                                            'is_tier_recycle', 'is_show_progress', 'max_unlock_tier_count_per_trax',
-                                           'created_by_username', 'modified_by_username']
+                                           'created_by_username', 'modified_by_username', 'loyalty_package']
     
     @property
     def completed_prograss_in_percentage(self):
@@ -967,7 +1010,7 @@ class MerchantScheduleProgram(BaseNModel, DictModel):
                                 parent                  = merchant_acct.create_ndb_key(),
                                 merchant_program        = merchant_program.create_ndb_key(),
                                 schedule_type           = schedule_type,
-                                program_configuration   = merchant_program.to_program_configuration(),
+                                program_configuration   = merchant_program.to_configuration(),
                                 start_date              = merchant_program.start_date,
                                 end_date                = merchant_program.end_date,
                                 
