@@ -16,10 +16,10 @@ from trexmodel import conf, program_conf
 from trexlib.utils.string_util import random_string
 from datetime import datetime, timedelta
 from trexmodel.utils.model.model_util import generate_transaction_id,\
-    string_to_key_property
+    string_to_key_property 
 from six import string_types
-from trexmodel.models.datastore.customer_models import Customer
 from flask_babel import gettext
+from trexmodel.models.datastore.customer_models import Customer
 
 logger = logging.getLogger('model')
 
@@ -56,6 +56,12 @@ class RewardEntitlement(BaseNModel, DictModel):
     def is_redeemed(self):
         return self.status == program_conf.REWARD_STATUS_REDEEMED
     
+    @property
+    def is_removed(self):
+        return self.status == program_conf.REWARD_STATUS_REMOVED
+    
+    
+    
 class CustomerEntitledReward(RewardEntitlement):
     '''
     Customer as ancestor
@@ -70,6 +76,10 @@ class CustomerEntitledReward(RewardEntitlement):
     def entitled_customer_key(self):
         return self.key.parent().urlsafe().decode('utf-8')
     
+    @property
+    def entitled_customer_acct(self):
+        return Customer.fetch(self.key.parent().urlsafe())
+    
     @property
     def reward_program_key(self):
         return self.reward_program.urlsafe().decode('utf-8')
@@ -115,13 +125,30 @@ class CustomerEntitledReward(RewardEntitlement):
         return cls.query(ndb.AND(cls.status==status), ancestor=customer.create_ndb_key()).fetch(limit=limit)
         
     @classmethod
-    def list_all_by_customer(cls, customer, limit = conf.MAX_FETCH_RECORD):
-        return cls.query(ancestor=customer.create_ndb_key()).fetch(limit=limit)    
+    def list_all_by_customer(cls, customer, offset=0, limit = conf.MAX_FETCH_RECORD):
+        return cls.query(ancestor=customer.create_ndb_key()).fetch(offset=offset, limit=limit)
+    
+    @classmethod
+    def list_all_by_user_acct(cls, user_acct, limit = conf.MAX_FETCH_RECORD):
+        return cls.query(ndb.AND(cls.user_acct==user_acct.create_ndb_key())).fetch(limit=limit)
+    
+    @classmethod
+    def delete_all_by_customer(cls, customer):
+        query = cls.query(ancestor=customer.create_ndb_key())
+        cls.delete_multiples(query)    
 
 class CustomerCountableReward(CustomerEntitledReward):
     reward_amount               = ndb.FloatProperty(required=True, default=0)
     used_reward_amount          = ndb.FloatProperty(required=True, default=0)
     
+    dict_properties     = [
+                            'reward_amount', 'used_reward_amount', 'transaction_id', 'rewarded_datetime', 
+                            'status',
+                           ]
+    
+    def __repr__(self):
+        return 'transaction_id=%s, rewarded_datetime=%s, reward_balance=%s'% (self.transaction_id, self.rewarded_datetime, self.reward_balance)
+    
     @property
     def is_reach_reward_limit(self):
         return self.status == program_conf.REWARD_STATUS_REACH_LIMIT
@@ -147,7 +174,15 @@ class CustomerCountableReward(CustomerEntitledReward):
             
         (result, next_cursor) = cls.list_all_with_condition_query(query, start_cursor=start_cursor, return_with_cursor=True, limit=limit)
         
-        return (result, next_cursor) 
+        return (result, next_cursor)
+    
+    @classmethod
+    def list_valid(cls, customer, limit=50):
+        query = cls.query(ndb.AND(cls.status==program_conf.REWARD_STATUS_VALID
+            ), ancestor=customer.create_ndb_key()).order(cls.expiry_date)
+            
+        return cls.list_all_with_condition_query(query, limit=limit)
+         
     
     @property
     def is_used(self):
@@ -287,25 +322,125 @@ class CustomerEntitledVoucher(CustomerEntitledReward):
     use_online                  = ndb.BooleanProperty(required=False, default=False)
     use_in_store                = ndb.BooleanProperty(required=False, default=False)
     
-    dict_properties     = ['redeem_code', 'voucher_configuration', 'rewarded_by_username', 'rewarded_datetime', 'transaction_id', 
-                           'invoice_id', 'merchant_reference_code', 'tags_list', 'status', 'is_reverted', 'is_used',
-                           'registered_outlet_key', 'registered_merchant_acct_key', 'registered_datetime', 'modified_datetime']
+    dict_properties     = [
+                            'redeem_code', 'configuration', 'rewarded_datetime', 'transaction_id', 
+                            'status', 'is_reverted', 'is_used',
+                           ]
     
-    def redeem(self, redeemed_by, redeemed_datetime=None):
+    @staticmethod
+    def list_by_transaction_id(transaction_id):
+        return CustomerEntitledVoucher.query(CustomerEntitledVoucher.transaction_id==transaction_id).fetch(limit=conf.MAX_FETCH_RECORD)
+    
+    @staticmethod
+    def list_redeemed_by_merchant_voucher(customer, merchant_voucher, passed_day_count=1):
+        
+        now = datetime.utcnow()
+        passed_datetime = now - timedelta(days = passed_day_count)
+        
+        result = CustomerEntitledVoucher.query(ndb.AND(
+                    CustomerEntitledVoucher.entitled_voucher==merchant_voucher.create_ndb_key(),
+                    CustomerEntitledVoucher.redeemed_datetime>=passed_datetime,
+                    CustomerEntitledVoucher.status==program_conf.REWARD_STATUS_REDEEMED,
+                    
+                    ), ancestor=customer.create_ndb_key()).fetch(limit=conf.MAX_FETCH_RECORD)
+    
+        
+        return result
+    
+    @staticmethod
+    def count_redeemed_by_merchant_voucher(customer, merchant_voucher, passed_day_count=1):
+        
+        now = datetime.utcnow()
+        passed_datetime = now - timedelta(days = passed_day_count)
+        
+        result = CustomerEntitledVoucher.query(ndb.AND(
+                    CustomerEntitledVoucher.entitled_voucher==merchant_voucher.create_ndb_key(),
+                    CustomerEntitledVoucher.redeemed_datetime>=passed_datetime,
+                    CustomerEntitledVoucher.status==program_conf.REWARD_STATUS_REDEEMED,
+                    
+                    ), ancestor=customer.create_ndb_key()).count(limit=conf.MAX_FETCH_RECORD)
+    
+        
+        return result
+    
+    @staticmethod
+    def count_redeemed_by_merchant_voucher_and_passed_redeemed_datetime(customer, merchant_voucher, passed_redeemed_datetime):
+        
+        result = CustomerEntitledVoucher.query(ndb.AND(
+                    CustomerEntitledVoucher.entitled_voucher==merchant_voucher.create_ndb_key(),
+                    CustomerEntitledVoucher.redeemed_datetime>=passed_redeemed_datetime,
+                    CustomerEntitledVoucher.status==program_conf.REWARD_STATUS_REDEEMED,
+                    
+                    ), ancestor=customer.create_ndb_key()).count(limit=conf.MAX_FETCH_RECORD)
+    
+        
+        return result
+    
+    def revert_from_redemption(self):
+        self.status = program_conf.REDEEM_STATUS_VALID
+        self.redeemed_datetime      = None
+        self.redeemed_by            = None
+        self.redeemed_by_username   = None
+        self.put()
+        
+    
+    def redeem(self, redeemed_by=None, redeemed_datetime=None):
         self.status = program_conf.REWARD_STATUS_REDEEMED   
         
         if redeemed_datetime is None:
-            redeemed_datetime = datetime.now()
+            redeemed_datetime = datetime.utcnow()
+        
+        redeemed_by_username = None
+        if redeemed_by:
+            redeemed_by_username = redeemed_by.username
         
         self.redeemed_datetime      = redeemed_datetime
         self.redeemed_by            = redeemed_by.create_ndb_key()
-        self.redeemed_by_username   = redeemed_by.username
+        self.redeemed_by_username   = redeemed_by_username
         self.put()
+    
+    def remove(self, removed_by=None, removed_datetime=None):
+        self.status = program_conf.REWARD_STATUS_REMOVED   
+        
+        if removed_datetime is None:
+            removed_datetime = datetime.utcnow()
+        
+        removed_by_username = None
+        if removed_by:
+            removed_by_username = removed_by.username
+            
+        self.removed_datetime      = removed_datetime
+        self.removed_by_username   = removed_by_username
+        self.put() 
+        
+    @property
+    def entitled_customer_entity(self):
+        return Customer.fetch(self.key.parent().urlsafe()) 
         
     @property
     def is_used(self):
         return self.status == program_conf.REWARD_STATUS_REDEEMED
     
+    @property
+    def is_valid_to_redeem(self):
+        if self.status == program_conf.REWARD_STATUS_VALID:
+            return True
+        return False
+    
+    def is_effective_to_redeem(self, checking_datetime=None):
+        if checking_datetime is None:
+            checking_datetime = datetime.utcnow().date()
+        if self.effective_date <= checking_datetime:
+            return True
+        return False
+    
+    def is_not_expired_to_redeem(self, checking_datetime=None):
+        if checking_datetime is None:
+            checking_datetime = datetime.utcnow().date()
+        if self.expiry_date >= checking_datetime:
+            return True
+        return False
+    
     @property
     def is_reverted(self):
         return self.status == program_conf.REWARD_STATUS_REVERTED
@@ -315,10 +450,10 @@ class CustomerEntitledVoucher(CustomerEntitledReward):
         return Outlet.fetch(self.transact_outlet.urlsafe())
     
     @property
-    def voucher_configuration(self):
+    def configuration(self):
         voucher = MerchantVoucher.fetch(self.entitled_voucher.urlsafe())
         if voucher:
-            return voucher.to_configuration()
+            return voucher.configuration
     
     @property
     def entitled_voucher_key(self):
@@ -336,6 +471,11 @@ class CustomerEntitledVoucher(CustomerEntitledReward):
     def merchant_voucher_key(self):
         return self.entitled_voucher.urlsafe().decode('utf-8')
     
+    def redeem_limit_configuration(self):
+        _redeem_limit_configuration = self.merchant_voucher_entity.redeem_limit_configuration
+        
+        return _redeem_limit_configuration
+    
     def to_redeem_info(self):
         return {
                 'redeem_code'       : self.redeem_code,
@@ -373,10 +513,13 @@ class CustomerEntitledVoucher(CustomerEntitledReward):
                                             
                                             rewarded_datetime       = rewarded_datetime,
                                             
+                                            user_acct               = customer_acct.registered_user_acct.create_ndb_key(),
                                             )
         
         customer_entiteld_voucher.put()
         
+        #customer_acct.update_after_added_voucher(customer_entiteld_voucher)
+        
         return customer_entiteld_voucher
         
         
@@ -388,11 +531,12 @@ class CustomerEntitledVoucher(CustomerEntitledReward):
     def list_by_customer(customer, limit=conf.MAX_FETCH_RECORD, offset=0, voucher_status=program_conf.REWARD_STATUS_VALID):
         return CustomerEntitledVoucher.query(ndb.AND(CustomerEntitledVoucher.status==voucher_status),
                 ancestor=customer.create_ndb_key()).fetch(limit=limit, offset=offset)
-
+    
+    '''
     @staticmethod
     def list_all_by_customer(customer, limit=conf.MAX_FETCH_RECORD, offset=0):
         return CustomerEntitledVoucher.query(ancestor=customer.create_ndb_key()).fetch(limit=limit, offset=offset)
-    
+    '''
         
     @staticmethod
     def update_customer_entiteld_voucher_summary(customer, voucher_status=program_conf.REWARD_STATUS_VALID):
@@ -408,11 +552,15 @@ class CustomerEntitledVoucher(CustomerEntitledReward):
                                             'redeem_code'       : customer_voucher.redeem_code,
                                             'effective_date'    : customer_voucher.effective_date.strftime('%d-%m-%Y'),
                                             'expiry_date'       : customer_voucher.expiry_date.strftime('%d-%m-%Y'),
+                                            'is_redeem'         : False,
                                             }    
                 if voucher_summary:
+                    entitled_voucher_summary[merchant_voucher_key]['key']=merchant_voucher_key
                     entitled_voucher_summary[merchant_voucher_key]['redeem_info_list'].append(new_redeem_info)
+                    
                 else:
                     entitled_voucher_summary[merchant_voucher_key]={
+                                                                    'key'               : merchant_voucher_key,
                                                                     'label'             : merchant_voucher.label,
                                                                     'image_url'         : merchant_voucher.image_public_url,
                                                                     'redeem_info_list'  : [new_redeem_info]
@@ -499,6 +647,10 @@ class CustomerEntitledTierRewardSummary(BaseNModel, DictModel):
                 'label' : tier_reward_program.label,
                 'tiers' :tier_details_list
                 }
+    
+    @staticmethod
+    def list_all_by_customer(customer, limit = conf.MAX_FETCH_RECORD, offset=0):
+        return CustomerEntitledTierRewardSummary.query(ancestor=customer.create_ndb_key()).fetch(offset=offset, limit=limit)
         
     @staticmethod
     def list_tier_reward_summary_by_customer(customer_acct):
@@ -537,7 +689,11 @@ class CustomerEntitledTierRewardSummary(BaseNModel, DictModel):
                             'unlock_condition'          : tier_setting.get('unlock_tier_condition'),
                             'unlock_condition_value'    : tier_setting.get('unlock_tier_condition_value'), 
                             'unlock_value'              : .0,
-                            }  
+                            'unlock_source_details'     : {},
+                            }
+            
+            
+              
             tier_reward_summary_list.append(tier_summary)  
             
         
@@ -555,17 +711,17 @@ class CustomerEntitledTierRewardSummary(BaseNModel, DictModel):
         return reward_summary
     
     @staticmethod
-    def update(customer_tier_reward_summary, tier_summary=None, cycle_start_datetime=None, cycle_end_datetime=None):
+    def update(customer_tier_reward_summary, tier_summary_list=None, cycle_start_datetime=None):
         if cycle_start_datetime:
             customer_tier_reward_summary.cycle_start_datetime = cycle_start_datetime
         
-        if cycle_end_datetime:
-            customer_tier_reward_summary.cycle_end_datetime = cycle_end_datetime
-            
-        customer_tier_reward_summary.tier_summary = tier_summary
         if customer_tier_reward_summary.is_cycle_completed:
             customer_tier_reward_summary.cycle_end_datetime = datetime.utcnow()
         
+        
+        customer_tier_reward_summary.tier_summary = tier_summary_list
+        
+        logger.debug('update CustomerEntitledTierRewardSummary with customer_tier_reward_summary=%s', customer_tier_reward_summary)
         customer_tier_reward_summary.put()
         
     @staticmethod
@@ -580,9 +736,10 @@ class CustomerEntitledTierRewardSummary(BaseNModel, DictModel):
         tier_details_list = customer_tier_reward_summary.tier_summary.get('tiers')
         
         for tier_details in tier_details_list:
-            tier_details['unlock_status']      = False
-            tier_details['unlock_datetime']    = None
-            tier_details['unlock_value']       = .0
+            tier_details['unlock_status']           = False
+            tier_details['unlock_datetime']         = None
+            tier_details['unlock_value']            = .0
+            tier_details['unlock_source_details']   = {}
         
         logger.debug('tier_details_list=%s', tier_details_list)
             
