@@ -3,10 +3,9 @@ Created on 20 Apr 2020
 
 @author: jacklok
 '''
-from trexmodel import conf as model_conf
+from trexmodel import conf as model_conf, conf
 from google.cloud import ndb
 from trexmodel.models.datastore.ndb_models import BaseNModel, DictModel
- 
 
 class Country(BaseNModel, DictModel):
     cnty_id                 = ndb.IntegerProperty(required=True)
@@ -38,7 +37,41 @@ class ContactUs(BaseNModel, DictModel):
                 
     @staticmethod
     def list(offset=0, limit=10):
-        return ContactUs.query().order(-ContactUs.created_datetime).fetch(offset=offset, limit=limit)    
+        return ContactUs.query().order(-ContactUs.created_datetime).fetch(offset=offset, limit=limit)
+
+class ContactBase(BaseNModel, DictModel):
+    company_name                = ndb.StringProperty(required=False)
+    name                        = ndb.StringProperty(required=True)
+    mobile_phone                = ndb.StringProperty(required=False)
+    email                       = ndb.StringProperty(required=True)
+    message                     = ndb.TextProperty(required=True)
+    created_datetime            = ndb.DateTimeProperty(required=True, auto_now=True)
+    
+    dict_properties = ['company_name', 'name', 'email', 'mobile_phone', 'message', 
+                       'created_datetime']
+    
+    @classmethod
+    def create(cls, name, company_name=None, email=None, mobile_phone=None, message=None):
+        cls(
+                name            = name,
+                company_name    = company_name,
+                email           = email,
+                mobile_phone    = mobile_phone,
+                message         = message,
+                ).put()
+                
+    @classmethod
+    def list(cls, offset=0, limit=10):
+        return cls.query().order(-cls.created_datetime).fetch(offset=offset, limit=limit)
+
+class ContactToUs(ContactBase):
+    pass
+    
+class JoinAsPartner(ContactBase):
+    pass
+
+class DemoRequest(ContactBase):
+    pass        
                 
 class Feedback(BaseNModel, DictModel):
     name                    = ndb.StringProperty(required=True)
@@ -158,9 +191,25 @@ class Tagging(BaseNModel, DictModel):
     @classmethod
     def get_by_label(cls, parent, label):
         return cls.query(ndb.AND(cls.label==label), ancestor=parent).fetch(limit=1)
+
+
+
+'''        
+class ScheduledProjectExecutionHistory(BaseNModel, DictModel):
+    merchant_acct       = ndb.KeyProperty(name="merchant_acct", kind=MerchantAcct)
+    merchant_program    = ndb.KeyProperty(name="merchant_program", kind=MerchantProgram)
+    created_datetime    = ndb.DateTimeProperty(required=True, auto_now_add=True)
     
-        
-        
+    @staticmethod
+    def create(merchant_acct, merchant_program):
+        ScheduledProjectExecutionHistory(
+                                    merchant_acct       = merchant_acct.create_ndb_key(),
+                                    merchant_program    = merchant_program.create_ndb_key(),
+                                    ).put()
+
+'''
+    
+            
     
     
     
\ No newline at end of file
