@@ -5,20 +5,20 @@ Created on 10 Apr 2020
 '''
 from google.cloud import ndb
 from trexmodel.models.datastore.ndb_models import BaseNModel, DictModel
-#from trexlib.utils.common.cache_util import cache
-from trexlib.utils.string_util import random_number, is_not_empty, random_string
+from trexlib.utils.string_util import random_number, is_not_empty, random_string,\
+    is_empty
 from flask_login import UserMixin
 import logging  
 from trexlib.utils.security_util import generate_user_id, hash_password
 
 from trexlib import conf as lib_conf
-from trexmodel import conf as model_conf
+from trexmodel import conf as model_conf, conf
 import json
 from json import JSONEncoder
-from flask_babel import gettext
 from trexlib.utils.common.date_util import to_day_of_year
 from datetime import datetime, timedelta
-
+from flask_babel import gettext
+from trexmodel.conf import USER_STATUS_ANONYMOUS, ACCOUNT_LOCKED_IN_MINUTES
 
 logger = logging.getLogger("model")
 
@@ -39,10 +39,15 @@ class UserMin(BaseNModel, DictModel, UserMixin):
     last_login_datetime         = ndb.DateTimeProperty(required=False)
 
     locked                      = ndb.BooleanProperty(required=False, default=False)
+    lock_expiry_datetime        = ndb.DateTimeProperty(required=False)
     active                      = ndb.BooleanProperty(required=True, default=True)
     try_count                   = ndb.IntegerProperty(required=False)
+    demo_account                = ndb.BooleanProperty(required=False, default=False)
+    deleted                     = ndb.BooleanProperty(required=False, default=False)
     
-    dict_properties             = ['user_id', 'name', 'email', 'gravatar_url', 'active', 'locked']
+    
+    
+    dict_properties             = ['user_id', 'name', 'email', 'gravatar_url', 'active', 'locked', 'lock_expiry_datetime']
     
     @classmethod
     #@cache.memoize(timeout=60)
@@ -74,6 +79,43 @@ class UserMin(BaseNModel, DictModel, UserMixin):
         
         return is_signin_password_valid
     
+    def set_reset_password_code(self, reset_password_code):
+        self.reset_password_code = reset_password_code
+        self.put()
+    
+    def change_password(self, new_password):
+        hashed_new_password = hash_password(self.user_id, new_password)
+        
+        logger.debug('change_password: new_password=%s', new_password)
+        logger.debug('change_password: hashed_new_password=%s', hashed_new_password)
+        
+        self.password = hashed_new_password 
+        self.put()
+        
+    def request_to_delete(self):
+        DeletedUser.create_from_user(self)    
+        #self.delete()
+        self.deleted = True
+        self.put()
+        
+    def add_try_count(self):
+        if self.try_count is None:
+            self.try_count = 0
+        
+        self.try_count +=1
+        if self.try_count>=5:
+            self.locked = True
+            self.try_count = 0
+            self.lock_expiry_datetime = datetime.utcnow() + timedelta(minutes=int(ACCOUNT_LOCKED_IN_MINUTES))
+        self.put()
+    
+    @property    
+    def is_still_lock(self):
+        if self.lock_expiry_datetime is not None and self.lock_expiry_datetime>datetime.utcnow():
+            return True
+        return False
+    
+    
 class Role(BaseNModel):
     id              = ndb.StringProperty(required=True)
     name            = ndb.StringProperty(required=True)
@@ -100,7 +142,8 @@ class UserBase(UserMin):
     birth_day_in_year           = ndb.IntegerProperty(required=False)
     gender                      = ndb.StringProperty(required=False, choices=set([
                                                                     model_conf.GENDER_MALE_CODE, 
-                                                                    model_conf.GENDER_FEMALE_CODE
+                                                                    model_conf.GENDER_FEMALE_CODE,
+                                                                    model_conf.GENDER_UNKNOWN_CODE
                                                                     ]))
     national_id                 = ndb.StringProperty(required=False, )
     
@@ -136,6 +179,8 @@ class UserBase(UserMin):
     request_reset_password_token                = ndb.StringProperty(required=False)
     reset_password_updated_datetime             = ndb.DateTimeProperty(required=False)
     
+    
+    
     @property
     def age(self):
         if self.birth_date:
@@ -153,11 +198,15 @@ class UserBase(UserMin):
     
     @classmethod
     def get_by_email(cls, email):
-        return User.query(ndb.AND(User.email==email)).get()
+        return User.query(ndb.AND(User.email==email, User.deleted==False)).get()
+    
+    @classmethod
+    def list_by_email(cls, email):
+        return User.query(ndb.AND(User.email==email)).fetch(limit=conf.MAX_FETCH_RECORD)
         
     @classmethod
     def get_by_mobile_phone(cls, mobile_phone):
-        return User.query(ndb.AND(User.mobile_phone==mobile_phone)).get()
+        return User.query(ndb.AND(User.mobile_phone==mobile_phone, User.deleted==False)).get()
     
     @classmethod
     def get_by_reset_password_token(cls, request_reset_password_token):
@@ -166,10 +215,17 @@ class UserBase(UserMin):
     @classmethod
     def create(cls, name=None, email=None, mobile_phone=None, 
                gender=None, birth_date=None, 
-               password=None, create_email_vc=False, create_mobile_phone_vc=False):
+               password=None, is_email_verified=True, 
+               is_mobile_phone_verified=False, 
+               reference_code=None, status=None
+               ):
         
         user_id             = generate_user_id()
-        reference_code      = random_number(16) 
+        if is_empty(reference_code):
+            reference_code      = random_number(16)
+        else:
+            reference_code = reference_code.replace("-", "")
+                 
         birth_date_date_str = None
         birth_day_in_year   = 0
         
@@ -182,20 +238,24 @@ class UserBase(UserMin):
         
         created_user = cls(user_id=user_id, name=name, email=email, mobile_phone=mobile_phone, 
                            gender=gender, birth_date=birth_date, birth_date_date_str=birth_date_date_str,  birth_day_in_year=birth_day_in_year,
-                           reference_code=reference_code
+                           reference_code=reference_code, status=status,
                            )
         
         hashed_password = hash_password(user_id, password)
         created_user.password = hashed_password
         
-        if create_email_vc:
+        
+        created_user.is_email_verified          = is_email_verified
+        created_user.is_mobile_phone_verified   = is_mobile_phone_verified
+        
+        if is_email_verified==False:
             created_user.email_vc                               = random_number(6)
             created_user.email_vc_expiry_datetime               = datetime.now() + timedelta(minutes=10)
         
-        if create_mobile_phone_vc:
+        if is_mobile_phone_verified==False:
             created_user.mobile_phone_vc                        = random_number(6)
-            created_user.mobile_phone_vc_expiry_datetime        = datetime.now() + timedelta(minutes=10)   
-            
+            created_user.mobile_phone_vc_expiry_datetime        = datetime.now() + timedelta(minutes=10)
+        
         created_user.put()
         
         return created_user
@@ -206,6 +266,7 @@ class UserBase(UserMin):
         
         mobile_phone    = kwargs.get('mobile_phone')
         password        = kwargs.get('password')
+        status          = kwargs.get('status')
         
         if mobile_phone:
             mobile_phone = mobile_phone.replace(" ", "")
@@ -254,10 +315,17 @@ class UserBase(UserMin):
         self.request_reset_password_expiry_datetime = self.request_reset_password_datetime + timedelta(minutes=5)
         self.put()
         
+    def set_reset_password_token(self, request_reset_password_token):
+        self.request_reset_password_token = request_reset_password_token
+        self.request_reset_password_datetime = datetime.utcnow()
+        self.request_reset_password_expiry_datetime = self.request_reset_password_datetime + timedelta(minutes=5)
+        self.put()
+        
     def reset_password(self, new_password):
         hashed_password = hash_password(self.user_id, new_password)
         self.password   = hashed_password
-        
+        self.try_count  = 0
+        self.locked     = False
         self.reset_password_updated_datetime = datetime.utcnow()
         self.put()    
         
@@ -273,22 +341,26 @@ class UserBase(UserMin):
     
     def reset_email_vc(self):
         self.email_vc = random_number(6)
-        self.email_vc_expiry_datetime = datetime.utcnow() + timedelta(minutes=10)
+        self.email_vc_expiry_datetime = datetime.utcnow() + timedelta(minutes=2)
         self.put() 
         
     def reset_mobile_phone_vc(self):
-        self.email_vc = random_number(6)
+        self.mobile_phone_vc = random_number(6)
+        #self.mobile_phone_vc_expiry_datetime = datetime.utcnow() + timedelta(minutes=10)
         self.mobile_phone_vc_expiry_datetime = datetime.utcnow() + timedelta(minutes=10)  
         self.put()   
         
 class User(UserBase):
-    redeem_pin               = ndb.StringProperty(required=False)
+    redeem_pin                 = ndb.StringProperty(required=False)
+    status                     = ndb.StringProperty(required=False, default=USER_STATUS_ANONYMOUS)
+    new_message_count          = ndb.IntegerProperty(required=False, default=0)
+    device_details             = ndb.JsonProperty()
     
     dict_properties            = [
                                     'mobile_phone', 'email', 'password', 'name', 'birth_date', 'gender',
                                     'reference_code', 'country', 'state', 'city', 'postcode', 
-                                    'address', 'redeem_pin'
-
+                                    'address', 'redeem_pin', 'new_message_count', 'status',
+                                    'device_details',
                                     ]
 
     #unique_attributes = 'email,username'
@@ -296,7 +368,7 @@ class User(UserBase):
     audit_properties            = [
                                     'mobile_phone', 'email', 'password', 'name', 'birth_date', 'gender',
                                     'reference_code', 'country', 'state', 'city', 'postcode', 
-                                    'address', 'redeem_pin'
+                                    'address', 'redeem_pin', 'status', 'device_details',
 
                                     ]
 
@@ -337,11 +409,95 @@ class User(UserBase):
         """Returns `True` if the user is active."""
         logger.info('calling is_active')
         return self.active
+    
+    def update_device_details(self, platform, device_token):
+        if self.device_details:
+            found_device_details_list_by_platform = self.device_details.get(platform)
+            if found_device_details_list_by_platform:
+                is_found = False
+                for device_details_by_platform in  found_device_details_list_by_platform:
+                    device_token_by_platform = device_details_by_platform.get('device_token')
+                    if device_token_by_platform:
+                        device_details_by_platform['last_updated_datetime'] = datetime.utcnow().strftime("%d-%m-%Y %H:%M:%S")
+                        is_found = True
+                        break
+                
+                if is_found == False:
+                    found_device_details_list_by_platform.append(
+                                                                {
+                                                                'device_token'              : device_token,
+                                                                'last_updated_datetime'     : datetime.utcnow().strftime("%d-%m-%Y %H:%M:%S"),
+                                                                }
+                                                            )
+                        
+                    
+            else:
+                self.device_details[platform] = [
+                                                    {
+                                                    'device_token'              : device_token,
+                                                    'last_updated_datetime'     : datetime.utcnow().strftime("%d-%m-%Y %H:%M:%S"),
+                                                    }
+                                                ]
+        else:
+            self.device_details = {
+                                    platform : [
+                                                {
+                                                    'device_token'              : device_token,
+                                                    'last_updated_datetime'     : datetime.utcnow().strftime("%d-%m-%Y %H:%M:%S"),
+                                                    }
+                                                ]
+                                }
+        self.put()
 
+class DeletedUser(User):
+    deleted_datetime            = ndb.DateTimeProperty(required=False, auto_now_add=True)
+    
+    
+    def __repr__(self):
+        return '''
+                DeletedUser[key=%s, 
+                name=%s, 
+                email=%s, 
+                mobile_phone=%s, 
+                country=%s, 
+                locked=%s,
+                deleted_datetime=%s
+                ]
+                ''' % (self.key_in_str, self.name, self.email, self.mobile_phone, self.country, self.locked, self.deleted_datetime)
+                
+    @property
+    def is_active(self):
+        return False
+    
+    @staticmethod
+    def create_from_user(user):
+        deleted_user = DeletedUser(
+                        
+                        user_id             = user.user_id,
+                        name                = user.name,
+                        email               = user.email,
+                        mobile_phone        = user.mobile_phone,
+                        gender              = user.gender,
+                        country             = user.country,
+                        birth_date          = user.birth_date, 
+                        birth_date_date_str = user.birth_date_date_str,  
+                        birth_day_in_year   = user.birth_day_in_year,
+                        reference_code      = user.reference_code,
+                        active              = False,
+                        )
+        
+        
+        '''
+        for key, value in vars(user).items():
+            setattr(deleted_user, key, value)
+        '''
+        deleted_user.put()
+        
+        logger.debug('deleted_user=%s', deleted_user)
       
 class LoggedInUser(UserMixin, DictModel):
     
-    def __init__(self, json_object, is_super_user=False, is_admin_user=False, is_merchant_user=False):
+    def __init__(self, json_object, is_super_user=False, is_admin_user=False, is_merchant_user=False, country=lib_conf.DEFAULT_COUNTRY_CODE):
         
         logging.debug('json_object=%s', json_object)
         
@@ -356,11 +512,12 @@ class LoggedInUser(UserMixin, DictModel):
         self.is_admin_user      = json_object.get('is_admin_user') or is_admin_user
         self.is_merchant_user   = json_object.get('is_merchant_user') or is_merchant_user
         self.permission         = json_object.get('permission')
+        self.country            = country
         
         self.show_key_in_dict   = False
         
         self.dict_properties     = ['user_id', 'name', 'gravatar_url', 'active', 'locked', 
-                                    'is_super_user', 'is_admin_user', 'is_merchant_user', 'permission']
+                                    'is_super_user', 'is_admin_user', 'is_merchant_user', 'permission', 'country']
         
     @property
     def is_authenticated(self):
